@@ -13,19 +13,19 @@
 static TLS int reccycle = 0;
 
 
-static void match_calc_add( double **scoringmtx, float *match, float **cpmx1, float **cpmx2, int i1, int lgth2, float **floatwork, int **intwork, int initialize )
+static void match_calc_add( double **scoringmtx, double *match, double **cpmx1, double **cpmx2, int i1, int lgth2, double **doublework, int **intwork, int initialize )
 {
 	int j, k, l;
-//	float scarr[26];
-	float **cpmxpd = floatwork;
+//	double scarr[26];
+	double **cpmxpd = doublework;
 	int **cpmxpdn = intwork;
 	int count = 0;
-	float *matchpt;
-	float **cpmxpdpt;
+	double *matchpt;
+	double **cpmxpdpt;
 	int **cpmxpdnpt;
 	int cpkd;
-	float *scarr;
-	scarr = calloc( nalphabets, sizeof( float ) );
+	double *scarr;
+	scarr = calloc( nalphabets, sizeof( double ) );
 	if( initialize )
 	{
 		for( j=0; j<lgth2; j++ )
@@ -53,9 +53,9 @@ static void match_calc_add( double **scoringmtx, float *match, float **cpmx1, fl
 			scarr[l] += scoringmtx[k][l] * cpmx1[i1][k];
 		}
 	}
-#if 0 /* ȤȤfloatworkΥȤդˤ */
+#if 0 /* ȤȤdoubleworkΥȤդˤ */
 	{
-		float *fpt, **fptpt, *fpt2;
+		double *fpt, **fptpt, *fpt2;
 		int *ipt, **iptpt;
 		fpt2 = match;
 		iptpt = cpmxpdn;
@@ -92,19 +92,19 @@ static void match_calc_add( double **scoringmtx, float *match, float **cpmx1, fl
 	free( scarr );
 }
 
-static void match_calc( double **n_dynamicmtx, float *match, float **cpmx1, float **cpmx2, int i1, int lgth2, float **floatwork, int **intwork, int initialize )
+static void match_calc( double **n_dynamicmtx, double *match, double **cpmx1, double **cpmx2, int i1, int lgth2, double **doublework, int **intwork, int initialize )
 {
 	int j, k, l;
-//	float scarr[26];
-	float **cpmxpd = floatwork;
+//	double scarr[26];
+	double **cpmxpd = doublework;
 	int **cpmxpdn = intwork;
 	int count = 0;
-	float *matchpt;
-	float **cpmxpdpt;
+	double *matchpt;
+	double **cpmxpdpt;
 	int **cpmxpdnpt;
 	int cpkd;
-	float *scarr;
-	scarr = calloc( nalphabets, sizeof( float ) );
+	double *scarr;
+	scarr = calloc( nalphabets, sizeof( double ) );
 	if( initialize )
 	{
 		for( j=0; j<lgth2; j++ )
@@ -132,9 +132,9 @@ static void match_calc( double **n_dynamicmtx, float *match, float **cpmx1, floa
 			scarr[l] += n_dynamicmtx[k][l] * cpmx1[i1][k];
 		}
 	}
-#if 0 /* ȤȤfloatworkΥȤդˤ */
+#if 0 /* ȤȤdoubleworkΥȤդˤ */
 	{
-		float *fpt, **fptpt, *fpt2;
+		double *fpt, **fptpt, *fpt2;
 		int *ipt, **iptpt;
 		fpt2 = match;
 		iptpt = cpmxpdn;
@@ -171,7 +171,7 @@ static void match_calc( double **n_dynamicmtx, float *match, float **cpmx1, floa
 	free( scarr );
 }
 
-static float Atracking( float *lasthorizontalw, float *lastverticalw,
+static double Atracking( double *lasthorizontalw, double *lastverticalw,
 						char **seq1, char **seq2, 
                         char **mseq1, char **mseq2, 
                         int **ijp, int icyc, int jcyc,
@@ -181,7 +181,7 @@ static float Atracking( float *lasthorizontalw, float *lastverticalw,
 	int i, j, l, iin, jin, ifi, jfi, lgth1, lgth2, k, klim;
 	char *gaptable1, *gt1bk;
 	char *gaptable2, *gt2bk;
-	float wm;
+	double wm;
 	lgth1 = ien-ist+1;
 	lgth2 = jen-jst+1;
 
@@ -320,46 +320,46 @@ static float Atracking( float *lasthorizontalw, float *lastverticalw,
 	return( 0.0 );
 }
 
-static float MSalignmm_tanni( double **n_dynamicmtx, int icyc, int jcyc, double *eff1, double *eff2, char **seq1, char **seq2, float **cpmx1, float **cpmx2, int ist, int ien, int jst, int jen, int alloclen, int fulllen1, int fulllen2, char **mseq1, char **mseq2, float **gapinfo, int headgp, int tailgp, float headgapfreq1_g, float headgapfreq2_g )
+static double MSalignmm_tanni( double **n_dynamicmtx, int icyc, int jcyc, double *eff1, double *eff2, char **seq1, char **seq2, double **cpmx1, double **cpmx2, int ist, int ien, int jst, int jen, int alloclen, int fulllen1, int fulllen2, char **mseq1, char **mseq2, double **gapinfo, int headgp, int tailgp, double headgapfreq1_g, double headgapfreq2_g )
 /* score no keisan no sai motokaraaru gap no atukai ni mondai ga aru */
 {
 //	int k;
 	register int i, j;
 	int ll1, ll2;
 	int lasti, lastj;
-	float wm = 0.0;   /* int ?????? */
-	float g;
-	float *currentw, *previousw;
+	double wm = 0.0;   /* int ?????? */
+	double g;
+	double *currentw, *previousw;
 #if USE_PENALTY_EX
-	float fpenalty_ex = (float)penalty_ex;
+	double fpenalty_ex = (double)penalty_ex;
 #endif
 #if 1
-	float *wtmp;
+	double *wtmp;
 	int *ijppt;
-	float *mjpt, *prept, *curpt;
+	double *mjpt, *prept, *curpt;
 	int *mpjpt;
 #endif
-	float mi, *m;
+	double mi, *m;
 	int **ijp;
 	int mpi, *mp;
-	float *w1, *w2;
-	float *initverticalw;    /* kufuu sureba iranai */
-	float *lastverticalw;    /* kufuu sureba iranai */
+	double *w1, *w2;
+	double *initverticalw;    /* kufuu sureba iranai */
+	double *lastverticalw;    /* kufuu sureba iranai */
 	int **intwork;
-	float **floatwork;
+	double **doublework;
 	int **intmtx;
 	int lgth1, lgth2;
-	float *ogcp1;
-	float *fgcp1;
-	float *ogcp2;
-	float *fgcp2;
-	float *gapfreq1f;
-	float *gapfreq2f;
+	double *ogcp1;
+	double *fgcp1;
+	double *ogcp2;
+	double *fgcp2;
+	double *gapfreq1f;
+	double *gapfreq2f;
 //	char **aseq1;
 //	char **aseq2;
 //	char **aseq1bk, **aseq2bk;
-	float headgapfreq1;
-	float headgapfreq2;
+	double headgapfreq1;
+	double headgapfreq2;
 
 
 	ogcp1 = gapinfo[0] + ist;
@@ -419,7 +419,7 @@ static float MSalignmm_tanni( double **n_dynamicmtx, int icyc, int jcyc, double
 	m = AllocateFloatVec( ll2+2 );
 	mp = AllocateIntVec( ll2+2 );
 
-	floatwork = AllocateFloatMtx( MAX( ll1, ll2 )+2, nalphabets+1 ); 
+	doublework = AllocateFloatMtx( MAX( ll1, ll2 )+2, nalphabets+1 ); 
 	intwork = AllocateIntMtx( MAX( ll1, ll2 )+2, nalphabets+1 ); 
 
 
@@ -430,9 +430,9 @@ static float MSalignmm_tanni( double **n_dynamicmtx, int icyc, int jcyc, double
 	currentw = w1;
 	previousw = w2;
 
-	match_calc( n_dynamicmtx, initverticalw, cpmx2+jst, cpmx1+ist, 0, lgth1, floatwork, intwork, 1 );
+	match_calc( n_dynamicmtx, initverticalw, cpmx2+jst, cpmx1+ist, 0, lgth1, doublework, intwork, 1 );
 
-	match_calc( n_dynamicmtx, currentw, cpmx1+ist, cpmx2+jst, 0, lgth2, floatwork, intwork, 1 );
+	match_calc( n_dynamicmtx, currentw, cpmx1+ist, cpmx2+jst, 0, lgth2, doublework, intwork, 1 );
 
 	if( headgp || ist != 0 )
 	{
@@ -468,7 +468,7 @@ static float MSalignmm_tanni( double **n_dynamicmtx, int icyc, int jcyc, double
 
 		previousw[0] = initverticalw[i-1];
 
-		match_calc( n_dynamicmtx, currentw, cpmx1+ist, cpmx2+jst, i, lgth2, floatwork, intwork, 0 );
+		match_calc( n_dynamicmtx, currentw, cpmx1+ist, cpmx2+jst, i, lgth2, doublework, intwork, 0 );
 		currentw[0] = initverticalw[i];
 
 		mi = previousw[0] + ogcp2[1] * gapfreq1f[i-1];
@@ -565,7 +565,7 @@ static float MSalignmm_tanni( double **n_dynamicmtx, int icyc, int jcyc, double
 	FreeIntVec( mp );
 
 
-	FreeFloatMtx( floatwork );
+	FreeFloatMtx( doublework );
 	FreeIntMtx( intwork );
 
 	FreeIntMtx( intmtx );
@@ -582,13 +582,13 @@ static float MSalignmm_tanni( double **n_dynamicmtx, int icyc, int jcyc, double
 }
 
 static void freearrays_rec1(
-	float *w1, float *w2, float *initverticalw, float *lastverticalw,
-	float *midw, float *midm, float *midn,
+	double *w1, double *w2, double *initverticalw, double *lastverticalw,
+	double *midw, double *midm, double *midn,
 	int *jumpbacki, int *jumpbackj, int *jumpforwi, int *jumpforwj, int *jumpdummi, int *jumpdummj,
-	float *m, int *mp,
-	float **floatwork, int **intwork
+	double *m, int *mp,
+	double **doublework, int **intwork
 #if STOREWM
-	, float **WMMTX, float **WMMTX2
+	, double **WMMTX, double **WMMTX2
 #endif
 )
 {
@@ -611,7 +611,7 @@ static void freearrays_rec1(
 	FreeFloatVec( m );
 	FreeIntVec( mp );
 
-	FreeFloatMtx( floatwork );
+	FreeFloatMtx( doublework );
 	FreeIntMtx( intwork );
 
 #if STOREWM
@@ -632,48 +632,48 @@ static void freearrays_rec2( char *gaps, char **aseq1, char **aseq2 )
 #endif
 }
 
-static float MSalignmm_rec( double **n_dynamicmtx, int icyc, int jcyc, double *eff1, double *eff2, char **seq1, char **seq2, float **cpmx1, float **cpmx2, int ist, int ien, int jst, int jen, int alloclen, int fulllen1, int fulllen2, char **mseq1, char **mseq2, int depth, float **gapinfo, int *chudanpt, int chudanref, int *chudanres, int headgp, int tailgp, float headgapfreq1_g, float headgapfreq2_g )
+static double MSalignmm_rec( double **n_dynamicmtx, int icyc, int jcyc, double *eff1, double *eff2, char **seq1, char **seq2, double **cpmx1, double **cpmx2, int ist, int ien, int jst, int jen, int alloclen, int fulllen1, int fulllen2, char **mseq1, char **mseq2, int depth, double **gapinfo, int *chudanpt, int chudanref, int *chudanres, int headgp, int tailgp, double headgapfreq1_g, double headgapfreq2_g )
 /* score no keisan no sai motokaraaru gap no atukai ni mondai ga aru */
 {
 //	int k;
 	int alnlen;
-	float value = 0.0;
+	double value = 0.0;
 	register int i, j;
 	char **aseq1, **aseq2;
 	int ll1, ll2, l, len;
 	int lasti, lastj, imid;
 	int jmid = 0; // by D.Mathog, a guess
-	float wm = 0.0;   /* int ?????? */
-	float g;
-	float *currentw, *previousw;
+	double wm = 0.0;   /* int ?????? */
+	double g;
+	double *currentw, *previousw;
 #if USE_PENALTY_EX
-	float fpenalty_ex = (float)penalty_ex;
+	double fpenalty_ex = (double)penalty_ex;
 #endif
-	float *wtmp;
+	double *wtmp;
 //	short *ijppt;
 	int *mpjpt;
 //	short **ijp;
 	int *mp;
 	int mpi;
-	float *mjpt, *prept, *curpt;
-	float mi;
-	float *m;
-	float *w1, *w2;
-//	float *match;
-	float *initverticalw;    /* kufuu sureba iranai */
-	float *lastverticalw;    /* kufuu sureba iranai */
+	double *mjpt, *prept, *curpt;
+	double mi;
+	double *m;
+	double *w1, *w2;
+//	double *match;
+	double *initverticalw;    /* kufuu sureba iranai */
+	double *lastverticalw;    /* kufuu sureba iranai */
 	int **intwork;
-	float **floatwork;
+	double **doublework;
 //	short **shortmtx;
 #if STOREWM
-	float **WMMTX;
-	float **WMMTX2;
+	double **WMMTX;
+	double **WMMTX2;
 #endif
-	float *midw;
-	float *midm;
-	float *midn;
+	double *midw;
+	double *midm;
+	double *midn;
 	int lgth1, lgth2;
-	float maxwm;
+	double maxwm;
 	int *jumpforwi;
 	int *jumpforwj;
 	int *jumpbacki;
@@ -683,20 +683,20 @@ static float MSalignmm_rec( double **n_dynamicmtx, int icyc, int jcyc, double *e
 	int jumpi, jumpj = 0; // by D.Mathog, a guess
 	char *gaps;
 	int ijpi, ijpj;
-	float *ogcp1;
-	float *fgcp1;
-	float *ogcp2;
-	float *fgcp2;
-	float firstm;
+	double *ogcp1;
+	double *fgcp1;
+	double *ogcp2;
+	double *fgcp2;
+	double firstm;
 	int firstmp;
 #if STOREWM
 	static TLS char ttt1[50000];
 	static TLS char ttt2[50000];
 #endif
-	float *gapfreq1f;
-	float *gapfreq2f;
-	float headgapfreq1;
-	float headgapfreq2;
+	double *gapfreq1f;
+	double *gapfreq2f;
+	double headgapfreq1;
+	double headgapfreq2;
 
 #if 0
 	int nglen1, nglen2;
@@ -827,7 +827,7 @@ static float MSalignmm_rec( double **n_dynamicmtx, int icyc, int jcyc, double *e
 	mp = AllocateIntVec( ll2+2 );
 	gaps = AllocateCharVec( MAX( ll1, ll2 ) + 2 );
 
-	floatwork = AllocateFloatMtx( MAX( ll1, ll2 )+2, nalphabets ); 
+	doublework = AllocateFloatMtx( MAX( ll1, ll2 )+2, nalphabets ); 
 	intwork = AllocateIntMtx( MAX( ll1, ll2 )+2, nalphabets ); 
 
 #if DEBUG
@@ -851,9 +851,9 @@ static float MSalignmm_rec( double **n_dynamicmtx, int icyc, int jcyc, double *e
 	currentw = w1;
 	previousw = w2;
 
-	match_calc( n_dynamicmtx, initverticalw, cpmx2+jst, cpmx1+ist, 0, lgth1, floatwork, intwork, 1 );
+	match_calc( n_dynamicmtx, initverticalw, cpmx2+jst, cpmx1+ist, 0, lgth1, doublework, intwork, 1 );
 
-	match_calc( n_dynamicmtx, currentw, cpmx1+ist, cpmx2+jst, 0, lgth2, floatwork, intwork, 1 );
+	match_calc( n_dynamicmtx, currentw, cpmx1+ist, cpmx2+jst, 0, lgth2, doublework, intwork, 1 );
 
 	for( i=1; i<lgth1+1; i++ )
 	{
@@ -908,7 +908,7 @@ static float MSalignmm_rec( double **n_dynamicmtx, int icyc, int jcyc, double *e
 				w1, w2, initverticalw, lastverticalw, midw, midm, midn,
 				jumpbacki, jumpbackj, jumpforwi, jumpforwj, jumpdummi, jumpdummj,
 				m, mp,
-				floatwork, intwork
+				doublework, intwork
 #if STOREWM
 				, WMMTX, WMMTX2
 #endif
@@ -923,7 +923,7 @@ static float MSalignmm_rec( double **n_dynamicmtx, int icyc, int jcyc, double *e
 
 		previousw[0] = initverticalw[i-1];
 
-		match_calc( n_dynamicmtx, currentw, cpmx1+ist, cpmx2+jst, i, lgth2, floatwork, intwork, 0 );
+		match_calc( n_dynamicmtx, currentw, cpmx1+ist, cpmx2+jst, i, lgth2, doublework, intwork, 0 );
 		currentw[0] = initverticalw[i];
 
 		m[0] = ogcp1[i];
@@ -1066,8 +1066,8 @@ static float MSalignmm_rec( double **n_dynamicmtx, int icyc, int jcyc, double *e
 
 // gyakudp
 
-	match_calc( n_dynamicmtx, initverticalw, cpmx2+jst, cpmx1+ist, lgth2-1, lgth1, floatwork, intwork, 1 );
-	match_calc( n_dynamicmtx, currentw, cpmx1+ist, cpmx2+jst, lgth1-1, lgth2, floatwork, intwork, 1 );
+	match_calc( n_dynamicmtx, initverticalw, cpmx2+jst, cpmx1+ist, lgth2-1, lgth1, doublework, intwork, 1 );
+	match_calc( n_dynamicmtx, currentw, cpmx1+ist, cpmx2+jst, lgth1-1, lgth2, doublework, intwork, 1 );
 
 	for( i=0; i<lgth1-1; i++ )
 	{
@@ -1135,7 +1135,7 @@ static float MSalignmm_rec( double **n_dynamicmtx, int icyc, int jcyc, double *e
 				w1, w2, initverticalw, lastverticalw, midw, midm, midn,
 				jumpbacki, jumpbackj, jumpforwi, jumpforwj, jumpdummi, jumpdummj,
 				m, mp,
-				floatwork, intwork
+				doublework, intwork
 #if STOREWM
 				, WMMTX, WMMTX2
 #endif
@@ -1149,7 +1149,7 @@ static float MSalignmm_rec( double **n_dynamicmtx, int icyc, int jcyc, double *e
 		currentw = wtmp;
 		previousw[lgth2-1] = initverticalw[i+1];
 //		match_calc( currentw, seq1, seq2, i, lgth2 );
-		match_calc( n_dynamicmtx, currentw, cpmx1+ist, cpmx2+jst, i, lgth2, floatwork, intwork, 0 );
+		match_calc( n_dynamicmtx, currentw, cpmx1+ist, cpmx2+jst, i, lgth2, doublework, intwork, 0 );
 
 		currentw[lgth2-1] = initverticalw[i];
 
@@ -1494,7 +1494,7 @@ static float MSalignmm_rec( double **n_dynamicmtx, int icyc, int jcyc, double *e
 		w1, w2, initverticalw, lastverticalw, midw, midm, midn,
 		jumpbacki, jumpbackj, jumpforwi, jumpforwj, jumpdummi, jumpdummj,
 		m, mp,
-		floatwork, intwork
+		doublework, intwork
 #if STOREWM
 		, WMMTX, WMMTX2
 #endif
@@ -1642,15 +1642,15 @@ static float MSalignmm_rec( double **n_dynamicmtx, int icyc, int jcyc, double *e
 }
 
 static void freearrays( 
-	float *ogcp1, 
-	float *ogcp2, 
-	float *fgcp1, 
-	float *fgcp2, 
-	float **cpmx1,
-	float **cpmx2, 
-	float *gapfreq1f,
-	float *gapfreq2f,
-	float **gapinfo, 
+	double *ogcp1, 
+	double *ogcp2, 
+	double *fgcp1, 
+	double *fgcp2, 
+	double **cpmx1,
+	double **cpmx2, 
+	double *gapfreq1f,
+	double *gapfreq2f,
+	double **gapinfo, 
 	char **mseq1, 
 	char **mseq2 
 )
@@ -1669,29 +1669,29 @@ static void freearrays(
 	FreeCharMtx( mseq2 );
 }
 
-float MSalignmm( double **n_dynamicmtx, char **seq1, char **seq2, double *eff1, double *eff2, int icyc, int jcyc, int alloclen, char *sgap1, char *sgap2, char *egap1, char *egap2, int *chudanpt, int chudanref, int *chudanres, int headgp, int tailgp )
+double MSalignmm( double **n_dynamicmtx, char **seq1, char **seq2, double *eff1, double *eff2, int icyc, int jcyc, int alloclen, char *sgap1, char *sgap2, char *egap1, char *egap2, int *chudanpt, int chudanref, int *chudanres, int headgp, int tailgp )
 /* score no keisan no sai motokaraaru gap no atukai ni mondai ga aru */
 {
 //	int k;
 	int i, j;
 	int ll1, ll2;
 	int lgth1, lgth2;
-	float wm = 0.0;   /* int ?????? */
+	double wm = 0.0;   /* int ?????? */
 	char **mseq1;
 	char **mseq2;
-	float *ogcp1;
-	float *ogcp2;
-	float *fgcp1;
-	float *fgcp2;
-	float **cpmx1;
-	float **cpmx2;
-	float **gapinfo;
-	float fpenalty = (float)penalty;
-	float *gapfreq1f;
-	float *gapfreq2f;
+	double *ogcp1;
+	double *ogcp2;
+	double *fgcp1;
+	double *fgcp2;
+	double **cpmx1;
+	double **cpmx2;
+	double **gapinfo;
+	double fpenalty = (double)penalty;
+	double *gapfreq1f;
+	double *gapfreq2f;
 	int nglen1, nglen2;
-	float headgapfreq1;
-	float headgapfreq2;
+	double headgapfreq1;
+	double headgapfreq2;
 
 #if 0
 	fprintf( stderr, "eff in SA+++align\n" );
@@ -1920,51 +1920,51 @@ float MSalignmm( double **n_dynamicmtx, char **seq1, char **seq2, double *eff1,
 //    variousdist
 // -------------------------------
 
-static void fillzero( float *s, int l )
+static void fillzero( double *s, int l )
 {
 	while( l-- ) *s++ = 0.0; 
 }
 
-static float MSalignmm_tanni_variousdist( double ***matrices, int icyc, int jcyc, char **seq1, char **seq2, float ***cpmx1s, float ***cpmx2s, int ist, int ien, int jst, int jen, int alloclen, int fulllen1, int fulllen2, char **mseq1, char **mseq2, float **gapinfo, int headgp, int tailgp, float headgapfreq1_g, float headgapfreq2_g )
+static double MSalignmm_tanni_variousdist( double ***matrices, int icyc, int jcyc, char **seq1, char **seq2, double ***cpmx1s, double ***cpmx2s, int ist, int ien, int jst, int jen, int alloclen, int fulllen1, int fulllen2, char **mseq1, char **mseq2, double **gapinfo, int headgp, int tailgp, double headgapfreq1_g, double headgapfreq2_g )
 /* score no keisan no sai motokaraaru gap no atukai ni mondai ga aru */
 {
 //	int k;
 	register int i, j, c;
 	int ll1, ll2;
 	int lasti, lastj;
-	float wm = 0.0;   /* int ?????? */
-	float g;
-	float *currentw, *previousw;
+	double wm = 0.0;   /* int ?????? */
+	double g;
+	double *currentw, *previousw;
 #if USE_PENALTY_EX
-	float fpenalty_ex = (float)penalty_ex;
+	double fpenalty_ex = (double)penalty_ex;
 #endif
 #if 1
-	float *wtmp;
+	double *wtmp;
 	int *ijppt;
-	float *mjpt, *prept, *curpt;
+	double *mjpt, *prept, *curpt;
 	int *mpjpt;
 #endif
-	float mi, *m;
+	double mi, *m;
 	int **ijp;
 	int mpi, *mp;
-	float *w1, *w2;
-	float *initverticalw;    /* kufuu sureba iranai */
-	float *lastverticalw;    /* kufuu sureba iranai */
+	double *w1, *w2;
+	double *initverticalw;    /* kufuu sureba iranai */
+	double *lastverticalw;    /* kufuu sureba iranai */
 	int ***intwork;
-	float ***floatwork;
+	double ***doublework;
 	int **intmtx;
 	int lgth1, lgth2;
-	float *ogcp1;
-	float *fgcp1;
-	float *ogcp2;
-	float *fgcp2;
-	float *gapfreq1f;
-	float *gapfreq2f;
+	double *ogcp1;
+	double *fgcp1;
+	double *ogcp2;
+	double *fgcp2;
+	double *gapfreq1f;
+	double *gapfreq2f;
 //	char **aseq1;
 //	char **aseq2;
 //	char **aseq1bk, **aseq2bk;
-	float headgapfreq1;
-	float headgapfreq2;
+	double headgapfreq1;
+	double headgapfreq2;
 
 
 	ogcp1 = gapinfo[0] + ist;
@@ -2024,7 +2024,7 @@ static float MSalignmm_tanni_variousdist( double ***matrices, int icyc, int jcyc
 	m = AllocateFloatVec( ll2+2 );
 	mp = AllocateIntVec( ll2+2 );
 
-	floatwork = AllocateFloatCub( maxdistclass, MAX( ll1, ll2 )+2, nalphabets+1 ); 
+	doublework = AllocateFloatCub( maxdistclass, MAX( ll1, ll2 )+2, nalphabets+1 ); 
 	intwork = AllocateIntCub( maxdistclass, MAX( ll1, ll2 )+2, nalphabets+1 ); 
 
 
@@ -2036,16 +2036,16 @@ static float MSalignmm_tanni_variousdist( double ***matrices, int icyc, int jcyc
 	previousw = w2;
 
 #if 0
-	match_calc( n_dynamicmtx, initverticalw, cpmx2+jst, cpmx1+ist, 0, lgth1, floatwork, intwork, 1 );
-	match_calc( n_dynamicmtx, currentw, cpmx1+ist, cpmx2+jst, 0, lgth2, floatwork, intwork, 1 );
+	match_calc( n_dynamicmtx, initverticalw, cpmx2+jst, cpmx1+ist, 0, lgth1, doublework, intwork, 1 );
+	match_calc( n_dynamicmtx, currentw, cpmx1+ist, cpmx2+jst, 0, lgth2, doublework, intwork, 1 );
 #else
 	fillzero( initverticalw, lgth1 );
 	for( c=0; c<maxdistclass; c++ )
-		match_calc_add( matrices[c], initverticalw, cpmx2s[c]+jst, cpmx1s[c]+ist, 0, lgth1, floatwork[c], intwork[c], 1 );
+		match_calc_add( matrices[c], initverticalw, cpmx2s[c]+jst, cpmx1s[c]+ist, 0, lgth1, doublework[c], intwork[c], 1 );
 
 	fillzero( currentw, lgth2 );
 	for( c=0; c<maxdistclass; c++ )
-		match_calc_add( matrices[c], currentw, cpmx1s[c]+ist, cpmx2s[c]+jst, 0, lgth2, floatwork[c], intwork[c], 1 );
+		match_calc_add( matrices[c], currentw, cpmx1s[c]+ist, cpmx2s[c]+jst, 0, lgth2, doublework[c], intwork[c], 1 );
 #endif
 
 	if( headgp || ist != 0 )
@@ -2083,11 +2083,11 @@ static float MSalignmm_tanni_variousdist( double ***matrices, int icyc, int jcyc
 		previousw[0] = initverticalw[i-1];
 
 #if 0
-		match_calc( n_dynamicmtx, currentw, cpmx1+ist, cpmx2+jst, i, lgth2, floatwork, intwork, 0 );
+		match_calc( n_dynamicmtx, currentw, cpmx1+ist, cpmx2+jst, i, lgth2, doublework, intwork, 0 );
 #else
 		fillzero( currentw, lgth2 );
 		for( c=0; c<maxdistclass; c++ )
-			match_calc_add( matrices[c], currentw, cpmx1s[c]+ist, cpmx2s[c]+jst, i, lgth2, floatwork[c], intwork[c], 0 );
+			match_calc_add( matrices[c], currentw, cpmx1s[c]+ist, cpmx2s[c]+jst, i, lgth2, doublework[c], intwork[c], 0 );
 #endif
 		currentw[0] = initverticalw[i];
 
@@ -2185,7 +2185,7 @@ static float MSalignmm_tanni_variousdist( double ***matrices, int icyc, int jcyc
 	FreeIntVec( mp );
 
 
-	FreeFloatCub( floatwork );
+	FreeFloatCub( doublework );
 	FreeIntCub( intwork );
 
 	FreeIntMtx( intmtx );
@@ -2202,13 +2202,13 @@ static float MSalignmm_tanni_variousdist( double ***matrices, int icyc, int jcyc
 }
 
 static void freearrays_rec1_variousdist(
-	float *w1, float *w2, float *initverticalw, float *lastverticalw,
-	float *midw, float *midm, float *midn,
+	double *w1, double *w2, double *initverticalw, double *lastverticalw,
+	double *midw, double *midm, double *midn,
 	int *jumpbacki, int *jumpbackj, int *jumpforwi, int *jumpforwj, int *jumpdummi, int *jumpdummj,
-	float *m, int *mp,
-	float ***floatwork, int ***intwork
+	double *m, int *mp,
+	double ***doublework, int ***intwork
 #if STOREWM
-	, float **WMMTX, float **WMMTX2
+	, double **WMMTX, double **WMMTX2
 #endif
 )
 {
@@ -2231,7 +2231,7 @@ static void freearrays_rec1_variousdist(
 	FreeFloatVec( m );
 	FreeIntVec( mp );
 
-	FreeFloatCub( floatwork );
+	FreeFloatCub( doublework );
 	FreeIntCub( intwork );
 
 #if STOREWM
@@ -2240,48 +2240,48 @@ static void freearrays_rec1_variousdist(
 #endif
 }
 
-static float MSalignmm_rec_variousdist( double ***matrices, int icyc, int jcyc, char **seq1, char **seq2, float ***cpmx1s, float ***cpmx2s, int ist, int ien, int jst, int jen, int alloclen, int fulllen1, int fulllen2, char **mseq1, char **mseq2, int depth, float **gapinfo, int *chudanpt, int chudanref, int *chudanres, int headgp, int tailgp, float headgapfreq1_g, float headgapfreq2_g )
+static double MSalignmm_rec_variousdist( double ***matrices, int icyc, int jcyc, char **seq1, char **seq2, double ***cpmx1s, double ***cpmx2s, int ist, int ien, int jst, int jen, int alloclen, int fulllen1, int fulllen2, char **mseq1, char **mseq2, int depth, double **gapinfo, int *chudanpt, int chudanref, int *chudanres, int headgp, int tailgp, double headgapfreq1_g, double headgapfreq2_g )
 /* score no keisan no sai motokaraaru gap no atukai ni mondai ga aru */
 {
 //	int k;
 	int alnlen;
-	float value = 0.0;
+	double value = 0.0;
 	register int i, j, c;
 	char **aseq1, **aseq2;
 	int ll1, ll2, l, len;
 	int lasti, lastj, imid;
 	int jmid = 0; // by D.Mathog, a guess
-	float wm = 0.0;   /* int ?????? */
-	float g;
-	float *currentw, *previousw;
+	double wm = 0.0;   /* int ?????? */
+	double g;
+	double *currentw, *previousw;
 #if USE_PENALTY_EX
-	float fpenalty_ex = (float)penalty_ex;
+	double fpenalty_ex = (double)penalty_ex;
 #endif
-	float *wtmp;
+	double *wtmp;
 //	short *ijppt;
 	int *mpjpt;
 //	short **ijp;
 	int *mp;
 	int mpi;
-	float *mjpt, *prept, *curpt;
-	float mi;
-	float *m;
-	float *w1, *w2;
-//	float *match;
-	float *initverticalw;    /* kufuu sureba iranai */
-	float *lastverticalw;    /* kufuu sureba iranai */
+	double *mjpt, *prept, *curpt;
+	double mi;
+	double *m;
+	double *w1, *w2;
+//	double *match;
+	double *initverticalw;    /* kufuu sureba iranai */
+	double *lastverticalw;    /* kufuu sureba iranai */
 	int ***intwork;
-	float ***floatwork;
+	double ***doublework;
 //	short **shortmtx;
 #if STOREWM
-	float **WMMTX;
-	float **WMMTX2;
+	double **WMMTX;
+	double **WMMTX2;
 #endif
-	float *midw;
-	float *midm;
-	float *midn;
+	double *midw;
+	double *midm;
+	double *midn;
 	int lgth1, lgth2;
-	float maxwm;
+	double maxwm;
 	int *jumpforwi;
 	int *jumpforwj;
 	int *jumpbacki;
@@ -2291,20 +2291,20 @@ static float MSalignmm_rec_variousdist( double ***matrices, int icyc, int jcyc,
 	int jumpi, jumpj = 0; // by D.Mathog, a guess
 	char *gaps;
 	int ijpi, ijpj;
-	float *ogcp1;
-	float *fgcp1;
-	float *ogcp2;
-	float *fgcp2;
-	float firstm;
+	double *ogcp1;
+	double *fgcp1;
+	double *ogcp2;
+	double *fgcp2;
+	double firstm;
 	int firstmp;
 #if STOREWM
 	static TLS char ttt1[50000];
 	static TLS char ttt2[50000];
 #endif
-	float *gapfreq1f;
-	float *gapfreq2f;
-	float headgapfreq1;
-	float headgapfreq2;
+	double *gapfreq1f;
+	double *gapfreq2f;
+	double headgapfreq1;
+	double headgapfreq2;
 
 #if 0
 	int nglen1, nglen2;
@@ -2435,7 +2435,7 @@ static float MSalignmm_rec_variousdist( double ***matrices, int icyc, int jcyc,
 	mp = AllocateIntVec( ll2+2 );
 	gaps = AllocateCharVec( MAX( ll1, ll2 ) + 2 );
 
-	floatwork = AllocateFloatCub( maxdistclass, MAX( ll1, ll2 )+2, nalphabets ); 
+	doublework = AllocateFloatCub( maxdistclass, MAX( ll1, ll2 )+2, nalphabets ); 
 	intwork = AllocateIntCub( maxdistclass, MAX( ll1, ll2 )+2, nalphabets ); 
 
 #if DEBUG
@@ -2460,16 +2460,16 @@ static float MSalignmm_rec_variousdist( double ***matrices, int icyc, int jcyc,
 	previousw = w2;
 
 #if 0
-	match_calc( n_dynamicmtx, initverticalw, cpmx2+jst, cpmx1+ist, 0, lgth1, floatwork, intwork, 1 );
-	match_calc( n_dynamicmtx, currentw, cpmx1+ist, cpmx2+jst, 0, lgth2, floatwork, intwork, 1 );
+	match_calc( n_dynamicmtx, initverticalw, cpmx2+jst, cpmx1+ist, 0, lgth1, doublework, intwork, 1 );
+	match_calc( n_dynamicmtx, currentw, cpmx1+ist, cpmx2+jst, 0, lgth2, doublework, intwork, 1 );
 #else
 	fillzero( initverticalw, lgth1 );
 	for( c=0; c<maxdistclass; c++ )
-		match_calc_add( matrices[c], initverticalw, cpmx2s[c]+jst, cpmx1s[c]+ist, 0, lgth1, floatwork[c], intwork[c], 1 );
+		match_calc_add( matrices[c], initverticalw, cpmx2s[c]+jst, cpmx1s[c]+ist, 0, lgth1, doublework[c], intwork[c], 1 );
 
 	fillzero( currentw, lgth2 );
 	for( c=0; c<maxdistclass; c++ )
-		match_calc_add( matrices[c], currentw, cpmx1s[c]+ist, cpmx2s[c]+jst, 0, lgth2, floatwork[c], intwork[c], 1 );
+		match_calc_add( matrices[c], currentw, cpmx1s[c]+ist, cpmx2s[c]+jst, 0, lgth2, doublework[c], intwork[c], 1 );
 #endif
 
 
@@ -2526,7 +2526,7 @@ static float MSalignmm_rec_variousdist( double ***matrices, int icyc, int jcyc,
 				w1, w2, initverticalw, lastverticalw, midw, midm, midn,
 				jumpbacki, jumpbackj, jumpforwi, jumpforwj, jumpdummi, jumpdummj,
 				m, mp,
-				floatwork, intwork
+				doublework, intwork
 #if STOREWM
 				, WMMTX, WMMTX2
 #endif
@@ -2542,11 +2542,11 @@ static float MSalignmm_rec_variousdist( double ***matrices, int icyc, int jcyc,
 		previousw[0] = initverticalw[i-1];
 
 #if 0
-		match_calc( n_dynamicmtx, currentw, cpmx1+ist, cpmx2+jst, i, lgth2, floatwork, intwork, 0 );
+		match_calc( n_dynamicmtx, currentw, cpmx1+ist, cpmx2+jst, i, lgth2, doublework, intwork, 0 );
 #else
 		fillzero( currentw, lgth2 );
 		for( c=0; c<maxdistclass; c++ )
-			match_calc_add( matrices[c], currentw, cpmx1s[c]+ist, cpmx2s[c]+jst, i, lgth2, floatwork[c], intwork[c], 0 );
+			match_calc_add( matrices[c], currentw, cpmx1s[c]+ist, cpmx2s[c]+jst, i, lgth2, doublework[c], intwork[c], 0 );
 #endif
 		currentw[0] = initverticalw[i];
 
@@ -2691,16 +2691,16 @@ static float MSalignmm_rec_variousdist( double ***matrices, int icyc, int jcyc,
 // gyakudp
 
 #if 0
-	match_calc( n_dynamicmtx, initverticalw, cpmx2+jst, cpmx1+ist, lgth2-1, lgth1, floatwork, intwork, 1 );
-	match_calc( n_dynamicmtx, currentw, cpmx1+ist, cpmx2+jst, lgth1-1, lgth2, floatwork, intwork, 1 );
+	match_calc( n_dynamicmtx, initverticalw, cpmx2+jst, cpmx1+ist, lgth2-1, lgth1, doublework, intwork, 1 );
+	match_calc( n_dynamicmtx, currentw, cpmx1+ist, cpmx2+jst, lgth1-1, lgth2, doublework, intwork, 1 );
 #else
 	fillzero( initverticalw, lgth1 );
 	for( c=0; c<maxdistclass; c++ )
-		match_calc_add( matrices[c], initverticalw, cpmx2s[c]+jst, cpmx1s[c]+ist, lgth2-1, lgth1, floatwork[c], intwork[c], 1 );
+		match_calc_add( matrices[c], initverticalw, cpmx2s[c]+jst, cpmx1s[c]+ist, lgth2-1, lgth1, doublework[c], intwork[c], 1 );
 
 	fillzero( currentw, lgth2 );
 	for( c=0; c<maxdistclass; c++ )
-		match_calc_add( matrices[c], currentw, cpmx1s[c]+ist, cpmx2s[c]+jst, lgth1-1, lgth2, floatwork[c], intwork[c], 1 );
+		match_calc_add( matrices[c], currentw, cpmx1s[c]+ist, cpmx2s[c]+jst, lgth1-1, lgth2, doublework[c], intwork[c], 1 );
 #endif
 
 	for( i=0; i<lgth1-1; i++ )
@@ -2769,7 +2769,7 @@ static float MSalignmm_rec_variousdist( double ***matrices, int icyc, int jcyc,
 				w1, w2, initverticalw, lastverticalw, midw, midm, midn,
 				jumpbacki, jumpbackj, jumpforwi, jumpforwj, jumpdummi, jumpdummj,
 				m, mp,
-				floatwork, intwork
+				doublework, intwork
 #if STOREWM
 				, WMMTX, WMMTX2
 #endif
@@ -2783,11 +2783,11 @@ static float MSalignmm_rec_variousdist( double ***matrices, int icyc, int jcyc,
 		currentw = wtmp;
 		previousw[lgth2-1] = initverticalw[i+1];
 #if 0
-		match_calc( n_dynamicmtx, currentw, cpmx1+ist, cpmx2+jst, i, lgth2, floatwork, intwork, 0 );
+		match_calc( n_dynamicmtx, currentw, cpmx1+ist, cpmx2+jst, i, lgth2, doublework, intwork, 0 );
 #else
 		fillzero( currentw, lgth2 );
 		for( c=0; c<maxdistclass; c++ )
-			match_calc_add( matrices[c], currentw, cpmx1s[c]+ist, cpmx2s[c]+jst, i, lgth2, floatwork[c], intwork[c], 0 );
+			match_calc_add( matrices[c], currentw, cpmx1s[c]+ist, cpmx2s[c]+jst, i, lgth2, doublework[c], intwork[c], 0 );
 #endif
 
 		currentw[lgth2-1] = initverticalw[i];
@@ -3133,7 +3133,7 @@ static float MSalignmm_rec_variousdist( double ***matrices, int icyc, int jcyc,
 		w1, w2, initverticalw, lastverticalw, midw, midm, midn,
 		jumpbacki, jumpbackj, jumpforwi, jumpforwj, jumpdummi, jumpdummj,
 		m, mp,
-		floatwork, intwork
+		doublework, intwork
 #if STOREWM
 		, WMMTX, WMMTX2
 #endif
@@ -3281,15 +3281,15 @@ static float MSalignmm_rec_variousdist( double ***matrices, int icyc, int jcyc,
 }
 
 static void freearrays_variousdist( 
-	float *ogcp1, 
-	float *ogcp2, 
-	float *fgcp1, 
-	float *fgcp2, 
-	float ***cpmx1s,
-	float ***cpmx2s, 
-	float *gapfreq1f,
-	float *gapfreq2f,
-	float **gapinfo, 
+	double *ogcp1, 
+	double *ogcp2, 
+	double *fgcp1, 
+	double *fgcp2, 
+	double ***cpmx1s,
+	double ***cpmx2s, 
+	double *gapfreq1f,
+	double *gapfreq2f,
+	double **gapinfo, 
 	char **mseq1, 
 	char **mseq2 
 )
@@ -3309,7 +3309,7 @@ static void freearrays_variousdist(
 }
 
 
-float MSalignmm_variousdist( double **pairoffset, double ***matrices, double **dummy_mtx, char **seq1, char **seq2, double *eff1, double *eff2, double **eff1s, double **eff2s, int icyc, int jcyc, int alloclen, char *sgap1, char *sgap2, char *egap1, char *egap2, int *chudanpt, int chudanref, int *chudanres, int headgp, int tailgp )
+double MSalignmm_variousdist( double **pairoffset, double ***matrices, double **dummy_mtx, char **seq1, char **seq2, double *eff1, double *eff2, double **eff1s, double **eff2s, int icyc, int jcyc, int alloclen, char *sgap1, char *sgap2, char *egap1, char *egap2, int *chudanpt, int chudanref, int *chudanres, int headgp, int tailgp )
 /* score no keisan no sai motokaraaru gap no atukai ni mondai ga aru */
 {
 //	fprintf( stderr, "IN MSalignmm_variousdist\n" );
@@ -3317,22 +3317,22 @@ float MSalignmm_variousdist( double **pairoffset, double ***matrices, double **d
 	int i, j, c;
 	int ll1, ll2;
 	int lgth1, lgth2;
-	float wm = 0.0;   /* int ?????? */
+	double wm = 0.0;   /* int ?????? */
 	char **mseq1;
 	char **mseq2;
-	float *ogcp1;
-	float *ogcp2;
-	float *fgcp1;
-	float *fgcp2;
-	float ***cpmx1s;
-	float ***cpmx2s;
-	float **gapinfo;
-	float fpenalty = (float)penalty;
-	float *gapfreq1f;
-	float *gapfreq2f;
+	double *ogcp1;
+	double *ogcp2;
+	double *fgcp1;
+	double *fgcp2;
+	double ***cpmx1s;
+	double ***cpmx2s;
+	double **gapinfo;
+	double fpenalty = (double)penalty;
+	double *gapfreq1f;
+	double *gapfreq2f;
 	int nglen1, nglen2;
-	float headgapfreq1;
-	float headgapfreq2;
+	double headgapfreq1;
+	double headgapfreq2;
 
 #if 0
 	fprintf( stderr, "eff in SA+++align\n" );
