@@ -1650,8 +1650,8 @@ double A__align_variousdist( int **which, double ***matrices, double **n_dynamic
 
 				if( c != which[i][j] )
 				{
-					masklist1[c] = realloc( masklist1[c], sizeof( int ) * nmask[c]+1 );
-					masklist2[c] = realloc( masklist2[c], sizeof( int ) * nmask[c]+1 );
+					masklist1[c] = realloc( masklist1[c], sizeof( int ) * (nmask[c]+1) );
+					masklist2[c] = realloc( masklist2[c], sizeof( int ) * (nmask[c]+1) );
 
 					masklist1[c][nmask[c]] = i;
 					masklist2[c][nmask[c]] = j;
@@ -2071,8 +2071,8 @@ for( i=0; i<lgth2; i++ )
 		if( chudanpt && *chudanpt != chudanref ) 
 		{
 //			fprintf( stderr, "\n\n## CHUUDAN!!! S\n" );
-			if( masklist1 ) FreeIntMtx( masklist1 ); masklist1 = NULL;
-			if( masklist2 ) FreeIntMtx( masklist2 ); masklist2 = NULL;
+			if( masklist1 ) freeintmtx( masklist1, maxdistclass ); masklist1 = NULL;
+			if( masklist2 ) freeintmtx( masklist2, maxdistclass ); masklist2 = NULL;
 			if( nmask ) free( nmask ); nmask = NULL;
 			*chudanres = 1;
 			return( -1.0 );
@@ -2175,8 +2175,8 @@ fprintf( stderr, "\n" );
 			if( chudanpt && *chudanpt != chudanref ) 
 			{
 //				fprintf( stderr, "\n\n## CHUUDAN!!! S\n" );
-				if( masklist1 ) FreeIntMtx( masklist1 ); masklist1 = NULL;
-				if( masklist2 ) FreeIntMtx( masklist2 ); masklist2 = NULL;
+				if( masklist1 ) freeintmtx( masklist1, maxdistclass ); masklist1 = NULL;
+				if( masklist2 ) freeintmtx( masklist2, maxdistclass ); masklist2 = NULL;
 				if( nmask ) free( nmask ); nmask = NULL;
 				*chudanres = 1;
 				return( -1.0 );
@@ -2353,8 +2353,8 @@ fprintf( stderr, "\n" );
 //	fprintf( stderr, "wm = %f\n", wm );
 
 
-	if( masklist1 ) FreeIntMtx( masklist1 ); masklist1 = NULL;
-	if( masklist2 ) FreeIntMtx( masklist2 ); masklist2 = NULL;
+	if( masklist1 ) freeintmtx( masklist1, maxdistclass ); masklist1 = NULL;
+	if( masklist2 ) freeintmtx( masklist2, maxdistclass ); masklist2 = NULL;
 	if( nmask ) free( nmask ); nmask = NULL;
 
 	return( wm );
