@@ -86,7 +86,7 @@ void profilealignment2( int n0, int n2, char **aln0, char **aln2, int alloclen,
 	if( alg == 'M' )
 		MSalignmm( n_dis_consweight_multi, aln0, aln2, effarr0, effarr2, n0, n2, alloclen, NULL, NULL, NULL, NULL, NULL, 0, NULL, 1, 1 ); //outgap=1, 2014/Dec/1
 	else
-		A__align( n_dis_consweight_multi, aln0, aln2, effarr0, effarr2, n0, n2, alloclen, NULL, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, 1, 1 ); //outgap=1, 2014/Dec/1
+		A__align( n_dis_consweight_multi, aln0, aln2, effarr0, effarr2, n0, n2, alloclen, NULL, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, 1, 1, -1, -1 ); //outgap=1, 2014/Dec/1
 
 	newlen = strlen( aln0[0] );
 
@@ -211,7 +211,7 @@ static void profilealignment( int n0, int n1, int n2, char **aln0, char **aln1,
 	if( alg == 'M' )
 		MSalignmm( n_dis_consweight_multi, aln0, aln2, effarr0, effarr2, n0, n2, alloclen, NULL, NULL, NULL, NULL, NULL, 0, NULL, 1, 1 ); //outgap=1, 2014/Dec/1
 	else
-		A__align( n_dis_consweight_multi, aln0, aln2, effarr0, effarr2, n0, n2, alloclen, NULL, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, 1, 1 ); //outgap=1, 2014/Dec/1
+		A__align( n_dis_consweight_multi, aln0, aln2, effarr0, effarr2, n0, n2, alloclen, NULL, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, 1, 1, -1, -1 ); //outgap=1, 2014/Dec/1
 
 	newlen = strlen( aln0[0] );
 
@@ -255,10 +255,10 @@ static void profilealignment( int n0, int n1, int n2, char **aln0, char **aln1,
 #endif
 
 
-//	free( effarr0 );
-//	free( effarr2 );
-//	free( allgap0 );
-//	free( allgap2 );
+	free( effarr0 );
+	free( effarr2 );
+	free( allgap0 );
+	free( allgap2 );
 }
 
 void eq2dashmatomete( char **s, int n )
