@@ -2217,10 +2217,17 @@ void *dndprethread( void *arg )
 		else
 			mtxv = maxdist * ( 1.0 - (double)naivepairscore11( seq[i], seq[j], penalty * 10  ) / bunbo );
 #if 1
-		if( mtxv > 9.0 || mtxv < 0.0 )
+		if( mtxv < 0.0 )
 		{
-			fprintf( stderr, "Distance %d-%d is strange, %f.\n", i, j, mtxv );
-			exit( 1 );
+			fprintf( stderr, "WARNING: distance %d-%d is strange, %f.\n", i, j, mtxv );
+			mtxv = 0.0;
+//			exit( 1 ); // 2016/Aug/3
+		}
+		if( mtxv > 9.9 )
+		{
+			fprintf( stderr, "WARNING: distance %d-%d is strange, %f.\n", i, j, mtxv );
+			mtxv = 9.9;
+//			exit( 1 ); // 2016/Aug/3
 		}
 #else // CHUUI!!!  2012/05/16
 		if( mtxv > 2.0 )
@@ -2666,10 +2673,17 @@ void dndpre( int nseq, char **seq, double **mtx ) // not used yet
 					mtxv = maxdist * ( 1.0 - (double)naivepairscore11( seq[i], seq[j], penalty * 10 ) / bunbo );
 
 #if 1
-				if( mtxv > 9.0 || mtxv < 0.0 )
+				if( mtxv < 0.0 )
 				{
-					fprintf( stderr, "Distance %d-%d is strange, %f.\n", i, j, mtxv );
-					exit( 1 );
+					fprintf( stderr, "WARNING: distance %d-%d is strange, %f.\n", i, j, mtxv );
+					mtxv = 0.0;
+//					exit( 1 ); // 2016/Aug/3
+				}
+				if( mtxv > 9.9 )
+				{
+					fprintf( stderr, "WARNING: distance %d-%d is strange, %f.\n", i, j, mtxv );
+					mtxv = 9.9;
+//					exit( 1 ); // 2016/Aug/3
 				}
 #else // CHUUI!!!  2012/05/16
 				if( mtxv > 2.0 )
