@@ -3,7 +3,7 @@
 #define DEFAULTOFS_B  -123   /* +10 -- -50  teido ka ? */
 
 
-void BLOSUMmtx( int n, double **matrix, double *freq, char *amino, char *amino_grp )
+void BLOSUMmtx( int n, double **matrix, double *freq, unsigned char *amino, char *amino_grp )
 {
 	/*
 	char locaminod[26] = "GASTPLIMVDNEQFYWKRHCXXX.-U";
