@@ -16,13 +16,13 @@ int njob, nlenmax;
 int amino_n[0x100];
 char amino_grp[0x100];
 //int amino_dis[0x100][0x100];
-int **amino_dis;
-double **n_disLN;
+int **amino_dis = NULL;
+double **n_disLN = NULL;
 //double amino_dis_consweight_multi[0x100][0x100];
-double **amino_dis_consweight_multi;
-int **n_dis;
-int **n_disFFT;
-double **n_dis_consweight_multi;
+double **amino_dis_consweight_multi = NULL;
+int **n_dis = NULL;
+int **n_disFFT = NULL;
+double **n_dis_consweight_multi = NULL;
 unsigned char amino[0x100];
 double polarity[0x100];
 double volume[0x100];
