@@ -6,10 +6,7 @@
 
 #define END_OF_VEC -1
 
-static int maxl;
-static int tsize;
 static char outputformat;
-static double lenfaca, lenfacb, lenfacc, lenfacd;
 static int nadd;
 #define PLENFACA 0.01
 #define PLENFACB 10000
@@ -133,7 +130,7 @@ void makecompositiontable_p( short *table, int *pointt )
 		table[point]++;
 }
 
-int commonsextet_p( short *table, int *pointt )
+static int localcommonsextet_p( short *table, int *pointt )
 {
 	int value = 0;
 	short tmp;
@@ -347,7 +344,7 @@ int main( int argc, char **argv )
 		if( !table1 ) ErrorExit( "Cannot allocate table1\n" );
 		makecompositiontable_p( table1, pointt[i] );
 
-		score = commonsextet_p( table1, pointt[i] );
+		score = localcommonsextet_p( table1, pointt[i] );
 		mtxself[i] = score;
 		free( table1 );
 	}
@@ -392,7 +389,7 @@ int main( int argc, char **argv )
 			lenfac = 1.0 / ( shorter / longer * lenfacd + lenfacb / ( longer + lenfacc ) + lenfaca );
 //			lenfac = 1.0;
 //			fprintf( stderr, "lenfac = %f (%.0f,%.0f)\n", lenfac, longer, shorter );
-			score = commonsextet_p( table1, pointt[j] );
+			score = localcommonsextet_p( table1, pointt[j] );
 			bunbo = MIN( mtxself[i], mtxself[j] );
 			if( outputformat == 'p' )
 			{
@@ -410,7 +407,7 @@ int main( int argc, char **argv )
 					fprintf( stdout, "%d-%d d=%4.2f l=%d,%d\n", i+1, j+1, ( 1.0 - score / bunbo ) * lenfac, nlen[i], nlen[j] );
 			}
 //			fprintf( stderr, "##### mtx = %f, mtx[i][0]=%f, mtx[j][0]=%f, bunbo=%f\n", mtx[i][j-i], mtx[i][0], mtx[j][0], bunbo );
-//          score = (double)commonsextet_p( table1, pointt[j] );
+//          score = (double)localcommonsextet_p( table1, pointt[j] );
 //			fprintf( stdout, "%d-%d d=%4.2f l=%d,%d\n", i+1, j+1, ( 1.0 - score / MIN( mtxself[i], mtxself[j] ) ) * 3, nlen[i], nlen[j] );
 
 
