@@ -187,7 +187,7 @@ void GroupAlign( int nseq1, int nseq2, char **name, int *nlen, char **seq, char
 	double *effarr1 = NULL;
 	double *effarr2 = NULL;
 	static char *indication1, *indication2;
-	double dumfl = 0.0;
+//	double dumfl = 0.0;
 	double dumdb = 0.0;
 	int intdum;
 #if DEBUG
@@ -255,7 +255,7 @@ void GroupAlign( int nseq1, int nseq2, char **name, int *nlen, char **seq, char
 		if( alg == 'M' )
 			pscore = MSalignmm( n_dis_consweight_multi, mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
 		else
-			pscore = A__align( n_dis_consweight_multi, mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, NULL, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
+			pscore = A__align( n_dis_consweight_multi, mseq1, mseq2, effarr1, effarr2, clus1, clus2, alloclen, NULL, &dumdb, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap, -1, -1 );
 	}
 #if DEBUG
 		time2 = getrusage_sec();
