@@ -4,17 +4,23 @@
 #define IODEBUG 0
 #define SCOREOUT 0
 
+#define GLOBAL 0
+
 #define END_OF_VEC -1
 
 int nadd;
 double thresholdtorev;
 int dodp;
 int addfragment;
+int mode = '2';
+int reflim = 1000;
+int contrastsort = 1;
 
 typedef struct _thread_arg
 {
 	int iend; 
 	char **seq;
+	int *map;
 	char *tmpseq;
 	int *res;
 	int **spointt;
@@ -27,7 +33,111 @@ typedef struct _thread_arg
 #endif
 } thread_arg_t;
 
+typedef struct _selfdpthread_arg
+{
+	int iend;
+	char **seq;
+	double *res;
+#ifdef enablemultithread
+	int *jshare;
+	int thread_no;
+	pthread_mutex_t *mutex_counter;
+#endif
+} selfdpthread_arg_t;
 
+typedef struct _contrast
+{
+	int pos; 
+	double dif;
+} contrastarr;
+
+static void	*selfdpthread( void *arg )
+{
+	selfdpthread_arg_t *targ = (selfdpthread_arg_t *)arg;
+	int iend = targ->iend;
+	char **seq = targ->seq;
+	double *res = targ->res;
+#ifdef enablemultithread
+	int thread_no = targ->thread_no;
+	int *jshare = targ->jshare; 
+#endif
+	int j;
+	char **revseq;
+
+	revseq = AllocateCharMtx( 1, nlenmax+1 );
+
+	j = -1;
+	while( 1 )
+	{
+#ifdef enablemultithread
+		if( nthread )
+		{
+			pthread_mutex_lock( targ->mutex_counter );
+			j = *jshare;
+			if( j%100 == 0 ) reporterr( "%d / %d (thread %d)   \r", j, iend, thread_no );
+			if( j == iend )
+			{
+				pthread_mutex_unlock( targ->mutex_counter );
+				break;
+			}
+			++(*jshare);
+			pthread_mutex_unlock( targ->mutex_counter );
+		}
+		else
+#endif
+		{
+			j++;
+			if( j%100 == 0 ) reporterr( "%d / %d      \r", j, iend );
+			if( j == iend ) 
+			{
+				break;
+			}
+		}
+
+		sreverse( revseq[0], seq[j] );
+#if GLOBAL
+		res[j] =  G__align11_noalign( n_dis_consweight_multi, penalty, penalty_ex, seq+j, seq+j, 0 );
+		res[j] -= G__align11_noalign( n_dis_consweight_multi, penalty, penalty_ex, seq+j, revseq, 0 );
+#else
+		res[j] =  L__align11_noalign( n_dis_consweight_multi, seq+j, seq+j );
+		res[j] -= L__align11_noalign( n_dis_consweight_multi, seq+j, revseq );
+#endif
+	}
+
+	creverse( 0 );
+	FreeCharMtx( revseq );
+#if GLOBAL
+	G__align11_noalign( NULL, 0, 0, NULL, NULL, 0 );
+#else
+	L__align11_noalign( NULL, NULL, NULL );
+#endif
+	return( NULL );
+}
+
+#if 0
+static void partshuffle( int size, int outsize, int *ary )
+{
+	int i;
+
+//	reporterr( "ary before shuffle = \n" );
+ //   for(i=0;i<size;i++) reporterr( "%d ", ary[i] );
+//	reporterr( "\n" );
+
+    for(i=0;i<outsize;i++)
+    {
+        int j = rand()%size;
+        int t = ary[i];
+        ary[i] = ary[j];
+        ary[j] = t;
+    }
+
+//	reporterr( "ary after shuffle = \n" );
+ //   for(i=0;i<outsize;i++) reporterr( "%d ", ary[i] );
+//	reporterr( "|" );
+ //   for(i=outsize;i<size;i++) reporterr( "%d ", ary[i] );
+//	reporterr( "\n" );
+}
+#endif
 
 void arguments( int argc, char *argv[] )
 {
@@ -49,7 +159,7 @@ void arguments( int argc, char *argv[] )
 	poffset = NOTSPECIFIED;
 	kimuraR = 2;
 	pamN = 200;
-	thresholdtorev = 0.1;
+	thresholdtorev = 0.0;
 	addfragment = 0;
 
 
@@ -106,6 +216,19 @@ void arguments( int argc, char *argv[] )
 					fprintf( stderr, "thresholdtorev = %f\n", thresholdtorev );
 					--argc; 
 					goto nextoption;
+				case 'o':
+					mode = *(*++argv);
+					fprintf( stderr, "mode = %c\n", mode );
+					--argc; 
+					goto nextoption;
+				case 'r':
+					reflim = myatoi(*++argv);
+					fprintf( stderr, "reflim = %d\n", reflim );
+					--argc; 
+					goto nextoption;
+				case 'c':
+					contrastsort = 0;
+					break;
 				case 'd':
 					dodp = 1;
 					break;
@@ -314,18 +437,145 @@ static int localcommonsextet_p2( short *table, int *pointt )
 	return( value );
 }
 
+static int compfunc( const void *a, const void *b )
+{
+	return ((contrastarr *)b)->dif - ((contrastarr *)a)->dif; // correct
+//	return ((contrastarr *)a)->dif - ((contrastarr *)b)->dif; // incorrect!
+} 
+
+static void makecontrastorder6mer( int *order, int **pointt, int **pointt_rev, char **seq, int iend, int shift )
+{
+	int i;
+	double *res;
+	contrastarr *arr;
+	short *table1, *table1_rev;
+
+
+	arr = calloc( iend, sizeof( contrastarr ) );
+	res = calloc( iend, sizeof( double ) );
+
+	for( i=0; i<iend; i++ )
+	{
+		if( i % 100 == 1 ) reporterr( "%d   \r", i );
+		table1 = (short *)calloc( tsize, sizeof( short ) );
+		if( !table1 ) ErrorExit( "Cannot allocate table1\n" );
+		makecompositiontable_p( table1, pointt[i] );
+		res[i] = localcommonsextet_p2( table1, pointt[i] );
+		free( table1 );
+
+		table1_rev = (short *)calloc( tsize, sizeof( short ) );
+		if( !table1_rev ) ErrorExit( "Cannot allocate table1\n" );
+		makecompositiontable_p( table1_rev, pointt_rev[i] );
+		res[i] -= localcommonsextet_p2( table1_rev, pointt[i] );
+		free( table1_rev );
+
+	}
+
+	for( i=0; i<iend; i++ )
+	{
+		arr[i].pos = i;
+		arr[i].dif = res[i];
+	}
+
+	qsort( arr, iend, sizeof( contrastarr ), compfunc );
+
+	for( i=0; i<iend; i++ )
+		order[i] = arr[i].pos + shift;
+
+//	for( i=0; i<iend; i++ ) reporterr( "%f\n", arr[i].dif );
+//	reporterr( "highest contrast, %s\n", seq[order[0]] );
+//	reporterr( "lowest contrast, %s\n", seq[order[iend-1]] );
+
+	free( arr );
+	free( res );
+
+}
+static void makecontrastorder( int *order, char **seq, int iend, int shift )
+{
+	int i;
+	double *res;
+	contrastarr *arr;
+
+	arr = calloc( iend, sizeof( contrastarr ) );
+	res = calloc( iend, sizeof( double ) );
+
+#ifdef enablemultithread
+	if( nthread )
+	{
+		int j;
+		pthread_t *handle;
+		pthread_mutex_t mutex_counter;
+		selfdpthread_arg_t *targ;
+		int *jsharept;
+		
+		targ = calloc( nthread, sizeof( selfdpthread_arg_t ) );
+		handle = calloc( nthread, sizeof( pthread_t ) );
+		pthread_mutex_init( &mutex_counter, NULL );
+		jsharept = calloc( 1, sizeof(int) );
+		*jsharept = 0;
+		
+		for( j=0; j<nthread; j++ )
+		{
+			targ[j].iend = iend;
+			targ[j].seq = seq;
+			targ[j].res = res; 
+			targ[j].jshare = jsharept;
+			targ[j].mutex_counter = &mutex_counter;
+			targ[j].thread_no = j;
+			pthread_create( handle+j, NULL, selfdpthread, (void *)(targ+j) );
+		}
+		for( j=0; j<nthread; j++ ) pthread_join( handle[j], NULL );
+		pthread_mutex_destroy( &mutex_counter );
+		free( handle );
+		free( targ );
+		free( jsharept );
+	}
+	else
+#endif
+	{
+		selfdpthread_arg_t *targ;
+		targ = calloc( 1, sizeof( selfdpthread_arg_t ) );
+		targ[0].iend = iend;
+		targ[0].seq = seq;
+		targ[0].res = res; 
+		selfdpthread( targ );
+		free( targ );
+	}
+
+	for( i=0; i<iend; i++ )
+	{
+		arr[i].pos = i;
+		arr[i].dif = res[i];
+	}
+
+	qsort( arr, iend, sizeof( contrastarr ), compfunc );
+
+	for( i=0; i<iend; i++ )
+		order[i] = arr[i].pos + shift;
+
+//	for( i=0; i<iend; i++ ) reporterr( "%f\n", arr[i].dif );
+//	reporterr( "highest contrast, %s\n", seq[order[0]] );
+//	reporterr( "lowest contrast, %s\n", seq[order[iend-1]] );
+
+	free( arr );
+	free( res );
+
+}
+
+
 static void	*directionthread( void *arg )
 {
 	thread_arg_t *targ = (thread_arg_t *)arg;
 	int iend = targ->iend;
 	char **seq = targ->seq;
+	int *map = targ->map;
 	char *tmpseq = targ->tmpseq;
 	int *res = targ->res;
 	int **spointt = targ->spointt;
 	short *table1 = targ->table1;
-	int iq = targ->iq;
+//	int iq = targ->iq;
 #ifdef enablemultithread
-	int thread_no = targ->thread_no;
+//	int thread_no = targ->thread_no;
 	int *jshare = targ->jshare; 
 #endif
 	int j;
@@ -348,7 +598,6 @@ static void	*directionthread( void *arg )
 			j = *jshare;
 			if( j == iend )
 			{
-				fprintf( stderr, "\r %d / %d (thread %d)   \r", iq, njob, thread_no );
 				pthread_mutex_unlock( targ->mutex_counter );
 				break;
 			}
@@ -361,7 +610,7 @@ static void	*directionthread( void *arg )
 			j++;
 			if( j == iend ) 
 			{
-				fprintf( stderr, "\r %d / %d  \r", iq, njob );
+//				if( iq%100==1 ) fprintf( stderr, "\r %d / %d  \r", iq, njob );
 				break;
 			}
 		}
@@ -372,19 +621,28 @@ static void	*directionthread( void *arg )
 //			strcpy( mseq1[0], tmpseq );
 //			strcpy( mseq2[0], seq[j] );
 			mseq1[0] = tmpseq;
-			mseq2[0] = seq[j];
-//			res[j] = G__align11_noalign( n_dis_consweight_multi, penalty, penalty_ex, mseq1, mseq2, 0 );
+			mseq2[0] = seq[map[j]];
+#if GLOBAL
+			res[j] = G__align11_noalign( n_dis_consweight_multi, penalty, penalty_ex, mseq1, mseq2, 0 );
+#else
 			res[j] = L__align11_noalign( n_dis_consweight_multi, mseq1, mseq2 );
+#endif
 		}
 		else
-			res[j] = localcommonsextet_p2( table1, spointt[j] );
+		{
+//			reporterr( "\n\nj=%d, map[j]=%d\n\n", j, map[j] );
+			res[j] = localcommonsextet_p2( table1, spointt[map[j]] );
+		}
 	}
 	if( dodp ) // nakuserukamo
 	{
 		free( mseq1 );
 		free( mseq2 );
-//		G__align11_noalign( NULL, 0, 0, NULL, NULL, 0 );
+#if GLOBAL
+		G__align11_noalign( NULL, 0, 0, NULL, NULL, 0 );
+#else
 		L__align11_noalign( NULL, NULL, NULL );
+#endif
 	}
 //	else
 //		if( nthread )  // inthread == 0 no toki free suru to, error. nazeda
@@ -397,7 +655,7 @@ int main( int argc, char *argv[] )
 	static int  *nlen;	
 	static int  *nogaplen;	
 	static char **name, **seq;
-	int i, j, istart, iend;
+	int i, j, istart, iend, ic;
 	FILE *infp;
 //	FILE *adfp;
 	char c;
@@ -407,9 +665,11 @@ int main( int argc, char *argv[] )
 	int  **pointt, **pointt_rev, **spointt;
 	double res_forward, res_reverse, res_max;
 	int ires, mres, mres2;
-	int *res;
+	int *res, *resr, *resf;
+	int *map;
 	static short *table1, *table1_rev;
 	static char **mseq1f, **mseq1r, **mseq2;
+	int *contrastorder;
 
 	arguments( argc, argv );
 #ifndef enablemultithread
@@ -479,7 +739,7 @@ int main( int argc, char *argv[] )
 	}
 
 	fprintf( stderr, "\n" );
-	if( alg == 'G' ) // do to the first sequence
+	if( alg == 'G' ) // dp to the first sequence
 	{
 		mseq1f = AllocateCharMtx( 1, nlenmax+nlenmax );
 		mseq1r = AllocateCharMtx( 1, nlenmax+nlenmax );
@@ -510,11 +770,13 @@ int main( int argc, char *argv[] )
 		{
 			gappick0( mseq2[0], seq[i] );
 
-//			res_forward = G__align11_noalign( n_dis_consweight_multi, penalty, penalty_ex, mseq1f, mseq2, 0 );
-//			res_reverse = G__align11_noalign( n_dis_consweight_multi, penalty, penalty_ex, mseq1r, mseq2, 0 );
-
+#if GLOBAL
+			res_forward = G__align11_noalign( n_dis_consweight_multi, penalty, penalty_ex, mseq1f, mseq2, 0 );
+			res_reverse = G__align11_noalign( n_dis_consweight_multi, penalty, penalty_ex, mseq1r, mseq2, 0 );
+#else
 			res_forward = L__align11_noalign( n_dis_consweight_multi, mseq1f, mseq2 );
 			res_reverse = L__align11_noalign( n_dis_consweight_multi, mseq1r, mseq2 );
+#endif
 #if 0
 
 			strcpy( mseq2[0], seq[i] );
@@ -560,6 +822,7 @@ int main( int argc, char *argv[] )
 	}
 	else if( alg == 'm' )
 	{
+
 		if( dodp ) // nakuserukamo
 		{
 			mseq1f = AllocateCharMtx( 1, nlenmax+1);
@@ -568,6 +831,7 @@ int main( int argc, char *argv[] )
 		}
 		else
 		{
+//			nthread = 0; // heiretsu keisan no kouritsu ha warui node
 			spointt = AllocateIntMtx( njob, 0 ); 
 			pointt = AllocateIntMtx( njob, nlenmax+1 );
 			pointt_rev = AllocateIntMtx( njob, nlenmax+1 );
@@ -576,6 +840,10 @@ int main( int argc, char *argv[] )
 	    revseq = AllocateCharVec( nlenmax+1 );
 		grpseq = AllocateIntVec( nlenmax+1 );
 		res = AllocateIntVec( njob );
+		resr = AllocateIntVec( njob );
+		resf = AllocateIntVec( njob );
+		map = AllocateIntVec( njob );
+		contrastorder = AllocateIntVec( njob );
 		if( dorp == 'd' ) tsize = (int)pow( 4, 6 );
 		else              tsize = (int)pow( 6, 6 ); // iranai
 
@@ -587,44 +855,128 @@ int main( int argc, char *argv[] )
 			if( nogaplen[i] > maxl ) maxl = nogaplen[i];
 		}
 
-		if( nadd )
-			iend = njob - nadd;
-		else
-			iend = 1;
+		reporterr( "Step 1/2\n" );
 
-		for( i=0; i<iend; i++ )
+		if( !dodp )
 		{
-//			fprintf( stdout, "%d, SKIP\n", i );
-			gappick0( tmpseq, seq[i] );
-			strcpy( seq[i], tmpseq );
-//			if( !nadd ) strcpy( seq[i], tmpseq ); // seq ha tsukawanaikara ii.
-
-			if( !dodp )
+			if( nadd )
+				iend = njob - nadd;
+			else
+				iend = 0; // keisan shinai
+	
+			for( i=0; i<iend; i++ )
 			{
+				gappick0( tmpseq, seq[i] );
+				strcpy( seq[i], tmpseq );
 				seq_grp_nuc( grpseq, tmpseq );
 				makepointtable_nuc( pointt[i], grpseq );
 				spointt[i] = pointt[i];
 			}
+	
+			if( nadd )
+				istart = njob - nadd;
+			else
+				istart = 0;
+			for( i=istart; i<njob; i++ ) 
+			{
 
-			strcpy( tmpseq, name[i] );
-			strcpy( name[i], "_F_" );
-			strncpy( name[i]+3, tmpseq+1, 10 );
-			name[i][13] = 0;
+				gappick0( tmpseq, seq[i] );
+				strcpy( seq[i], tmpseq );
+				sreverse( revseq, tmpseq );
+
+				seq_grp_nuc( grpseq, tmpseq );
+				makepointtable_nuc( pointt[i], grpseq );
+//				makecompositiontable_p( table1, pointt[i] ); -> moto no basho ni modosu
+				seq_grp_nuc( grpseq, revseq );
+				makepointtable_nuc( pointt_rev[i], grpseq );
+//				makecompositiontable_p( table1_rev, pointt_rev[i] ); -> moto no basho ni modosu
+				spointt[i] = pointt[i];
+
+
+//				reporterr( "pointt[i] = %p\n", pointt[i] );
+//				reporterr( "pointt[i][0] = %p\n", pointt[i][0] );
+
+			}	
 		}
 
+
+		if( contrastsort ) // sukoshi chuui
+		{
+
+
+			if( nadd )
+			{
+				iend = njob-nadd;
+				for( i=0; i<iend; i++ ) contrastorder[i] = i;
+				istart = njob-nadd;
+				iend = nadd;
+			}
+			else
+			{
+				istart = 0;
+				iend = njob;
+			}
+
+			if( dodp )
+				makecontrastorder( contrastorder+istart, seq+istart, iend, istart );
+			else
+				makecontrastorder6mer( contrastorder+istart, pointt+istart, pointt_rev+istart, seq+istart, iend, istart );
+		}
+		else
+		{
+			for( i=0; i<njob; i++ ) contrastorder[i] = i;
+		}
+
+
+//		reporterr( "contrastorder = \n" );
+//		for( i=0; i<njob; i++ )
+//			reporterr( "%d ", contrastorder[i] );
+//		reporterr( "\n" );
+
+
+
+		if( nadd )
+			iend = njob - nadd;
+		else
+			iend = 1;
+		for( i=0; i<iend; i++ )
+		{
+			ic = contrastorder[i];
+//			fprintf( stdout, "%d, SKIP\n", i );
+			gappick0( tmpseq, seq[ic] );
+			strcpy( seq[ic], tmpseq );
+//			if( !nadd ) strcpy( seq[i], tmpseq ); // seq ha tsukawanaikara ii.
+
+#if 0 // -> makecontrastorder() no mae ni idou
+			if( !dodp )
+			{
+				seq_grp_nuc( grpseq, tmpseq );
+				makepointtable_nuc( pointt[ic], grpseq );
+				spointt[ic] = pointt[ic];
+			}
+#endif
+
+			strcpy( tmpseq, name[ic] );
+			strcpy( name[ic], "_F_" );
+			strncpy( name[ic]+3, tmpseq+1, 10 );
+			name[ic][13] = 0;
+		}
+
+		reporterr( "\n\nStep 2/2\n" );
+
 		if( nadd )
 			istart = njob - nadd;
 		else
 			istart = 1;
-
-		fprintf( stderr, "\n" );
 		for( i=istart; i<njob; i++ ) 
 		{
 //			fprintf( stderr, "\r %d / %d ", i, njob );
-			gappick0( tmpseq, seq[i] );
-			strcpy( seq[i], tmpseq );
+			ic = contrastorder[i];
+			gappick0( tmpseq, seq[ic] );
+			strcpy( seq[ic], tmpseq );
 			sreverse( revseq, tmpseq );
 
+#if 0 // -> makecontrastorder() no mae ni idou
 			if( !dodp )
 			{
 				table1 = (short *)calloc( tsize, sizeof( short ) );
@@ -632,12 +984,23 @@ int main( int argc, char *argv[] )
 				table1_rev = (short *)calloc( tsize, sizeof( short ) );
 				if( !table1_rev ) ErrorExit( "Cannot allocate table1_rev\n" );
 				seq_grp_nuc( grpseq, tmpseq );
-				makepointtable_nuc( pointt[i], grpseq );
-				makecompositiontable_p( table1, pointt[i] );
+				makepointtable_nuc( pointt[ic], grpseq );
+				makecompositiontable_p( table1, pointt[ic] );
 				seq_grp_nuc( grpseq, revseq );
-				makepointtable_nuc( pointt_rev[i], grpseq );
-				makecompositiontable_p( table1_rev, pointt_rev[i] );
+				makepointtable_nuc( pointt_rev[ic], grpseq );
+				makecompositiontable_p( table1_rev, pointt_rev[ic] );
 			}
+#else
+			if( !dodp )
+			{
+				table1 = (short *)calloc( tsize, sizeof( short ) );
+				if( !table1 ) ErrorExit( "Cannot allocate table1\n" );
+				table1_rev = (short *)calloc( tsize, sizeof( short ) );
+				if( !table1_rev ) ErrorExit( "Cannot allocate table1_rev\n" );
+				makecompositiontable_p( table1, pointt[ic] );
+				makecompositiontable_p( table1_rev, pointt_rev[ic] );
+			}
+#endif
 
 			if( nadd && addfragment )
 				iend = njob-nadd;
@@ -645,6 +1008,28 @@ int main( int argc, char *argv[] )
 				iend = i;
 
 
+			if( iend > reflim ) 
+			{
+//				reporterr( "iend = %d -> %d\n", iend, reflim );
+#if 0
+				for( j=0; j<iend; j++ ) map[j] = j;
+				partshuffle( iend, reflim, map );
+#else
+				for( j=0; j<iend; j++ ) map[j] = contrastorder[j];
+#endif
+				iend = reflim; // approximation
+			}
+			else
+			{
+#if 0
+				for( j=0; j<iend; j++ ) map[j] = j;
+#else
+				for( j=0; j<iend; j++ ) map[j] = contrastorder[j];
+#endif
+			}
+
+//			reporterr( "reflim = %d, seq[%d] = %s\n", reflim, contrastorder[0], seq[contrastorder[0]] );
+
 #ifdef enablemultithread
 			if( nthread )
 			{
@@ -659,16 +1044,18 @@ int main( int argc, char *argv[] )
 				jsharept = calloc( 1, sizeof(int) );
 				*jsharept = 0;
 		
+				if( i%100==1 ) fprintf( stderr, " %d / %d (%d threads)   \r", i, njob, nthread );
 				for( j=0; j<nthread; j++ )
 				{
 					targ[j].iend = iend;
+					targ[j].map = map;
 					targ[j].seq = seq;
 					targ[j].tmpseq = tmpseq; 
-					targ[j].res = res; 
+					targ[j].res = resf; 
 					targ[j].spointt = spointt; 
 					targ[j].table1 = table1; 
 					targ[j].jshare = jsharept;
-					targ[j].iq = i;
+					targ[j].iq = i; // iranai
 					targ[j].mutex_counter = &mutex_counter;
 					targ[j].thread_no = j;
 					pthread_create( handle+j, NULL, directionthread, (void *)(targ+j) );
@@ -683,37 +1070,22 @@ int main( int argc, char *argv[] )
 #endif
 			{
 				thread_arg_t *targ;
+
+				if( i%100==1 ) fprintf( stderr, " %d / %d   \r", i, njob );
 				targ = calloc( 1, sizeof( thread_arg_t ) );
 				targ[0].iend = iend;
+				targ[0].map = map;
 				targ[0].seq = seq;
 				targ[0].tmpseq = tmpseq; 
-				targ[0].res = res; 
+				targ[0].res = resf; 
 				targ[0].spointt = spointt; 
 				targ[0].table1 = table1; 
-				targ[0].iq = i; 
+				targ[0].iq = i;  // iranai
 				directionthread( targ );
 				free( targ );
 			}
 
 
-			mres = mres2 = 0;
-			for( j=0; j<iend; j++ )
-			{
-				ires = res[j];
-//				fprintf( stdout, "ires (%d,%d) = %d\n", i, j, ires );
-//				fflush( stdout );
-				if( ires>mres2 ) 
-				{
-					if( ires>mres ) 
-					{
-						mres2 = mres;
-						mres = ires;
-					}
-					else
-						mres2 = ires;
-				}
-			}
-			res_forward = (double)( mres + mres2 ) / 2;
 
 #ifdef enablemultithread
 			if( nthread )
@@ -733,12 +1105,13 @@ int main( int argc, char *argv[] )
 				{
 					targ[j].iend = iend;
 					targ[j].seq = seq;
+					targ[j].map = map;
 					targ[j].tmpseq = revseq; 
-					targ[j].res = res; 
+					targ[j].res = resr; 
 					targ[j].spointt = spointt; 
 					targ[j].table1 = table1_rev; 
 					targ[j].jshare = jsharept;
-					targ[j].iq = i;
+					targ[j].iq = i; // iranai
 					targ[j].mutex_counter = &mutex_counter;
 					targ[j].thread_no = j;
 					pthread_create( handle+j, NULL, directionthread, (void *)(targ+j) );
@@ -756,55 +1129,108 @@ int main( int argc, char *argv[] )
 				targ = calloc( 1, sizeof( thread_arg_t ) );
 				targ[0].iend = iend;
 				targ[0].seq = seq;
+				targ[0].map = map;
 				targ[0].tmpseq = revseq; 
-				targ[0].res = res; 
+				targ[0].res = resr; 
 				targ[0].spointt = spointt;
 				targ[0].table1 = table1_rev; 
-				targ[0].iq = i; 
+				targ[0].iq = i;  // iranai
 				directionthread( targ );
 				free( targ );
 			}
 
-			mres = mres2 = 0;
-			for( j=0; j<iend; j++ )
+			if( mode == '2' )
 			{
-				ires = res[j];
-				if( ires>mres2 )
+				mres = mres2 = 0;
+				for( j=0; j<iend; j++ )
 				{
-					if( ires>mres ) 
+					ires = resf[j];
+//					fprintf( stdout, "ires (%d,%d) = %d\n", i, j, ires );
+//					fflush( stdout );
+					if( ires>mres2 ) 
 					{
-						mres2 = mres;
-						mres = ires;
+						if( ires>mres ) 
+						{
+							mres2 = mres;
+							mres = ires;
+						}
+						else
+							mres2 = ires;
 					}
-					else
-						mres2 = ires;
 				}
+				res_forward = (double)( mres + mres2 ) / 2;
+				mres = mres2 = 0;
+				for( j=0; j<iend; j++ )
+				{
+					ires = resr[j];
+					if( ires>mres2 )
+					{
+						if( ires>mres ) 
+						{
+							mres2 = mres;
+							mres = ires;
+						}
+						else
+							mres2 = ires;
+					}
+				}
+				res_reverse = (double)( mres + mres2 ) / 2;
+				res_max = MAX(res_reverse,res_forward);
 			}
-			res_reverse = (double)( mres + mres2 ) / 2;
-
-//			fprintf( stdout, "\n" );
-//			fprintf( stdout, "score_for(%d,%d) = %f\n", 0, i, res_forward );
-//			fprintf( stdout, "score_rev(%d,%d) = %f\n", 0, i, res_reverse );
-//			fflush( stdout );
-			res_max = MAX(res_reverse,res_forward);
-			if( (res_reverse-res_forward)/res_max > thresholdtorev ) // tekitou
-
+//			reporterr( "i=%d, res_reverse = %f\n", i, res_reverse );
+			else if( mode == '1' )
 			{
-				strcpy( seq[i], revseq );
+				res_reverse = 0.0;
+				for( j=0; j<iend; j++ ) if( res_reverse < (double)resr[j] ) res_reverse = (double)resr[j];
+				res_forward = 0.0;
+				for( j=0; j<iend; j++ ) if( res_forward < (double)resf[j] ) res_forward = (double)resf[j];
+				res_max = 1.0;
+			}
 
-				strcpy( tmpseq, name[i] );
-				strcpy( name[i], "_R_" );
-				strncpy( name[i]+3, tmpseq+1, 10 );
-				name[i][13] = 0;
-				if( !dodp ) spointt[i] = pointt_rev[i];
+			else if( mode == 'd' )
+			{
+				res_reverse = 0.0;
+				for( j=0; j<iend; j++ ) if( res_reverse < (double)(resr[j]-resf[j]) ) res_reverse = (double)(resr[j]-resf[j]);
+				res_forward = 0.0;
+				for( j=0; j<iend; j++ ) if( res_forward < (double)(resf[j]-resr[j]) ) res_forward = (double)(resf[j]-resr[j]);
+				res_max = 1.0;
+			}
+
+			else if( mode == 'a' )
+			{
+				res_reverse = 0.0;
+				for( j=0; j<iend; j++ ) res_reverse += (double)resr[j];
+				res_reverse /= (double)iend;
+				res_forward = 0.0;
+				for( j=0; j<iend; j++ ) res_forward += (double)resf[j];
+				res_forward /= (double)iend;
+				res_max = 1.0;
 			}
 			else
 			{
-				strcpy( tmpseq, name[i] );
-				strcpy( name[i], "_F_" );
-				strncpy( name[i]+3, tmpseq+1, 10 );
-				name[i][13] = 0;
-				if( !dodp ) spointt[i] = pointt[i];
+				reporterr( "Unknown mode!\n" );
+				exit( 1 );
+			}
+
+
+			if( (res_reverse>res_forward) ) // tekitou
+//			if( (res_reverse-res_forward)/res_max > thresholdtorev ) // tekitou
+			{
+				strcpy( seq[ic], revseq );
+
+				strcpy( tmpseq, name[ic] );
+				strcpy( name[ic], "_R_" );
+				strncpy( name[ic]+3, tmpseq+1, 10 );
+				name[ic][13] = 0;
+				if( !dodp ) spointt[ic] = pointt_rev[ic];
+			}
+			else
+			{
+				strcpy( tmpseq, name[ic] );
+				strcpy( name[ic], "_F_" );
+				strncpy( name[ic]+3, tmpseq+1, 10 );
+				name[ic][13] = 0;
+				if( !dodp ) spointt[ic] = pointt[ic];
 			}
 
 			if( !dodp )
@@ -814,10 +1240,28 @@ int main( int argc, char *argv[] )
 			}
 		}
 
-		free( grpseq );
+		if( name[0][1] == 'R' )
+		{
+			for( j=0; j<njob; j++ ) 
+			{
+				if( name[j][1] == 'R' ) 
+					name[j][1] = 'F';
+				else
+					name[j][1] = 'R';
+			}
+		}
+
+		creverse( 0 );
 		free( tmpseq );
 		free( revseq );
+		free( grpseq );
 		free( res );
+		free( resr );
+		free( resf );
+		free( map );
+		free( nlen );
+		free( nogaplen );
+		free( contrastorder );
 		if( dodp )
 		{
 			FreeCharMtx( mseq1f );
@@ -844,6 +1288,11 @@ int main( int argc, char *argv[] )
 		fprintf( stdout, "%s\n", name[i] );
 	}
 
+	FreeCharMtx( seq );
+	FreeCharMtx( name );
+	freeconstants();
+	closeFiles();
+
 	fprintf( stderr, "\n" );
 	SHOWVERSION;
 	return( 0 );
