@@ -431,6 +431,13 @@ int main( int argc, char *argv[] )
 	for( j=0; j<nlenmax; j++ ) free( alnpairprob[j] );
 	free( alnpairprob );
 	free( alnpairnum );
+	free( order );
+	free( nlen );
+	FreeCharMtx( seq );
+	FreeCharMtx( nogap );
+	FreeCharMtx( name );
+	FreeIntMtx( gapmap );
+	freeconstants();
 	fprintf( stderr, "%d thread(s)\n", nthread );
 	return( 0 );
 
