@@ -206,7 +206,7 @@ static void pairalign( char **name, int nlen[M], char **seq, double *effarr, int
 
 
 			fprintf( stderr, "adding %d-%d\r", i, j );
-			putlocalhom2( pseq[0], pseq[1], localhomtable[i]+j, 0, 0, (int)pscore, strlen( pseq[0] ) );
+			putlocalhom2( pseq[0], pseq[1], localhomtable[i]+j, 0, 0, (int)pscore, strlen( pseq[0] ), 'k' );
 			for( tmpptr=localhomtable[i]+j; tmpptr; tmpptr=tmpptr->next )
 			{
 				if( tmpptr->opt == -1.0 ) continue;
