@@ -2046,7 +2046,8 @@ static void *athread( void *arg ) // alg='R', alg='r' -> tsukawarenai.
 		pthread_mutex_unlock( targ->mutex_counter );
 
 
-		if( j == i+1 || j % 100 == 0 ) 
+//		if( j == i+1 || j % 100 == 0 ) 
+		if( j == i+1 && i % 10 == 0 ) 
 		{
 			fprintf( stderr, "% 5d / %d (by thread %3d) \r", i, njob-nadd, thread_no );
 //			fprintf( stderr, "% 5d - %5d / %d (thread %d)\n", i, j, njob, thread_no );
