@@ -1492,8 +1492,8 @@ double partA__align_variousdist( int **which, double ***matrices, double **n_dyn
 
 				if( c != which[i][j] )
 				{
-					masklist1[c] = realloc( masklist1[c], sizeof( int ) * nmask[c]+1 );
-					masklist2[c] = realloc( masklist2[c], sizeof( int ) * nmask[c]+1 );
+					masklist1[c] = realloc( masklist1[c], sizeof( int ) * (nmask[c]+1) );
+					masklist2[c] = realloc( masklist2[c], sizeof( int ) * (nmask[c]+1) );
 
 					masklist1[c][nmask[c]] = i;
 					masklist2[c][nmask[c]] = j;
@@ -1843,8 +1843,8 @@ for( i=0; i<lgth2; i++ )
 		{
 //			fprintf( stderr, "\n\n## CHUUDAN!!! i\n" );
 			*chudanres = 1;
-			if( masklist1 ) FreeIntMtx( masklist1 ); masklist1 = NULL;
-			if( masklist2 ) FreeIntMtx( masklist2 ); masklist2 = NULL;
+			if( masklist1 ) freeintmtx( masklist1, maxdistclass ); masklist1 = NULL;
+			if( masklist2 ) freeintmtx( masklist2, maxdistclass ); masklist2 = NULL;
 			if( nmask ) free( nmask ); nmask = NULL;
 			return( -1.0 );
 		}
@@ -1926,8 +1926,8 @@ fprintf( stderr, "\n" );
 			{
 //				fprintf( stderr, "\n\n## CHUUDAN!!! j\n" );
 				*chudanres = 1;
-				if( masklist1 ) FreeIntMtx( masklist1 ); masklist1 = NULL;
-				if( masklist2 ) FreeIntMtx( masklist2 ); masklist2 = NULL;
+				if( masklist1 ) freeintmtx( masklist1, maxdistclass ); masklist1 = NULL;
+				if( masklist2 ) freeintmtx( masklist2, maxdistclass ); masklist2 = NULL;
 				if( nmask ) free( nmask ); nmask = NULL;
 				return( -1.0 );
 			}
@@ -2096,8 +2096,8 @@ fprintf( stderr, "\n" );
 	for( j=0; j<jcyc; j++ ) fprintf( stderr, "%s\n", mseq2[j] );
 	*/
 
-	if( masklist1 ) FreeIntMtx( masklist1 ); masklist1 = NULL;
-	if( masklist2 ) FreeIntMtx( masklist2 ); masklist2 = NULL;
+	if( masklist1 ) freeintmtx( masklist1, maxdistclass ); masklist1 = NULL;
+	if( masklist2 ) freeintmtx( masklist2, maxdistclass ); masklist2 = NULL;
 	if( nmask ) free( nmask ); nmask = NULL;
 
 	return( wm );
