@@ -3,6 +3,7 @@
 #define DEBUG 0
 
 char *alignmentfile;
+int keeplength;
 
 static void fillorichar( int nseq, int *oripos, char **a, char **s )
 {
@@ -10,6 +11,7 @@ static void fillorichar( int nseq, int *oripos, char **a, char **s )
 	char *pta, *pts;
 	for( i=0; i<nseq; i++ )
 	{
+
 		pta = a[i];
 		pts = s[oripos[i]];
 		while( *pta )
@@ -18,14 +20,14 @@ static void fillorichar( int nseq, int *oripos, char **a, char **s )
 			if( *pta++ == 0 )
 			{
 				fprintf( stderr, "ERROR!!\n" );
-				fprintf( stderr, "alignment is inconsistent with the original sequences\n" );
+				fprintf( stderr, "alignment is inconsistent with the original sequences (1)\n" );
 				exit( 1 );
 			}
 		}
 		if( *pts != 0 )
 		{
 			fprintf( stderr, "ERROR!!\n" );
-			fprintf( stderr, "alignment is inconsistent with the original sequences\n" );
+			fprintf( stderr, "alignment is inconsistent with the original sequences (2)\n" );
 			exit( 1 );
 		}
 	}
@@ -35,6 +37,8 @@ void arguments( int argc, char *argv[] )
 {
     int c;
 
+	keeplength = 0;
+
     while( --argc > 0 && (*++argv)[0] == '-' )
 	{
         while ( (c = *++argv[0]) )
@@ -49,6 +53,13 @@ void arguments( int argc, char *argv[] )
 					alignmentfile = *++argv;
 					--argc;
 					goto nextoption;
+				case 'Z': // add2ndharfarg wo tsukau tame.
+					break;
+				case 'p': // add2ndharfarg wo tsukau tame.
+					break;
+				case 'Y':
+					keeplength = 1;
+					break;
                 default:
                     fprintf( stderr, "illegal option %c\n", c );
                     argc = 0;
@@ -71,6 +82,7 @@ int main( int argc, char *argv[] )
 {
 	FILE *infp;
 	FILE *alfp;
+	FILE *dlfp;
 	char **name;
 	char **aname;
 	char **oname;
@@ -82,6 +94,9 @@ int main( int argc, char *argv[] )
 	int i, o, prelen;
 	int nlenmin;
 	int njobs, njoba;
+//	int **dlist;
+//	int *ndel;
+	char *gett;
 
 	arguments( argc, argv );
 
@@ -111,6 +126,16 @@ int main( int argc, char *argv[] )
 		fprintf( stderr, "No alignment is given.\n" );
 		exit( 1 );
 	}
+
+	if( keeplength )
+	{
+		dlfp = fopen( "_deletelist", "r" );
+		if( !dlfp )
+		{
+			fprintf( stderr, "Cannot open _deletefile\n" );
+			exit( 1 );
+		}
+	}
 	
 	dorp = NOTSPECIFIED;
 	getnumlen_casepreserve( infp, &nlenmin );
@@ -122,6 +147,7 @@ int main( int argc, char *argv[] )
 	nlen = AllocateIntVec( njob );
 	oripos = AllocateIntVec( njob );
 	readData_pointer_casepreserve( infp, name, nlen, seq );
+	fclose( infp );
 
 	dorp = NOTSPECIFIED;
 	getnumlen( alfp );
@@ -131,9 +157,42 @@ int main( int argc, char *argv[] )
 	aname = AllocateCharMtx( njob, B+1 );
 	oname = AllocateCharMtx( njob, B+1 );
 	readData_pointer( alfp, aname, nlen, aseq );
+	fclose( alfp );
 
 	for( i=0; i<njob; i++ ) gappick_samestring( seq[i] );
 
+//	reporterr( "seq[njob-1] = %s\n", seq[njob-1] );
+
+	if( keeplength )
+	{
+		gett = calloc( 1000, sizeof( char ) );
+//		ndel = (int *)calloc( njob, sizeof( int ) );
+//		for( i=0; i<njob; i++ ) ndel[i] = 0;
+//		dlist = (int **)calloc( njob+1, sizeof( int * ) );
+//		for( i=0; i<njob; i++ ) dlist[i] = NULL;
+//		dlist[njob] = NULL;
+
+		while( 1 )
+		{
+			fgets( gett, 999, dlfp );
+			if( feof( dlfp ) ) break;
+			sscanf( gett, "%d %d", &i, &o );
+//			reporterr( "%d, %d\n", i, o );
+//			dlist[i] = realloc( dlist[i], sizeof( int ) * (ndel[i]+2) );
+//			dlist[i][ndel[i]] = o;
+//			ndel[i]++;
+
+			seq[i][o] = '-';
+
+		}
+		fclose( dlfp );
+	}
+
+	for( i=0; i<njob; i++ )
+	{
+		gappick_samestring( seq[i] );
+	}
+
 	if( njoba != njobs )
 	{
 		fprintf( stderr, "ERROR!!\n" );
@@ -222,7 +281,9 @@ int main( int argc, char *argv[] )
 			fprintf( stderr, "In input file,\n" );
 			fprintf( stderr, "name[%d] = %s\n", o, name[o] );
 			fprintf( stderr, "but in alignment file,\n" );
+			fprintf( stderr, "oname[%d] = %s\n", i, oname[i] );
 			fprintf( stderr, "npt0 = %s\n", npt0 );
+			fprintf( stderr, "prelen = %d\n", prelen );
 			fprintf( stderr, "name[%d] = %s\n", i, aname[i] );
 			exit( 1 );
 		}
