@@ -586,6 +586,7 @@ void arguments( int argc, char *argv[] )
 	tokyoripara = NOTSPECIFIED;
 	legacygapcost = 0;
 	nwildcard = 0;
+	outnumber = 0;
 
     while( --argc > 0 && (*++argv)[0] == '-' )
 	{
@@ -659,6 +660,9 @@ void arguments( int argc, char *argv[] )
 				case 'l':
 					uselongest = 0;
 					break;
+				case 'n' :
+					outnumber = 1;
+					break;
 #if 1
 				case 'a':
 					fmodel = 1;
