@@ -52,6 +52,8 @@ typedef struct _threadarg
 	int *finishpt;
 	int **skipthisbranch;
 	double **distmtx;
+	int ntarget;
+	int *targetmap;
 	pthread_mutex_t *mutex;
 	pthread_cond_t *collection_end;
 	pthread_cond_t *collection_start;
@@ -260,6 +262,7 @@ static void freelocalarrays(
 	int **memlist,
 	char *pairbuf,
 	LocalHom *** localhomshrink,
+	char *swaplist,
 	double **smalldistmtx,
 	double ***scoringmatrices,
 	double **eff1s, double **eff2s,
@@ -319,6 +322,7 @@ static void freelocalarrays(
 			free( localhomshrink[i] ); // nakamimo??
 		}
 		free( localhomshrink );
+		free( swaplist );
 	}
 }
 
@@ -356,6 +360,8 @@ static void *athread( void *arg )
 	int *finishpt = targ->finishpt;
 	int **skipthisbranch = targ->skipthisbranch;
 	double **distmtx = targ->distmtx;
+	int ntarget = targ->ntarget;
+	int *targetmap = targ->targetmap;
 
 	int i, k, l, ii;
 	double gain;
@@ -375,6 +381,7 @@ static void *athread( void *arg )
 	RNApair ***grouprna1, ***grouprna2;
 	RNApair *rnapairboth;
 	LocalHom ***localhomshrink;
+	char *swaplist;
 	int *gapmap1, *gapmap2;
 	double tscore, mscore;
 	double oimpmatchdouble;
@@ -480,8 +487,10 @@ static void *athread( void *arg )
 
 	if( rnakozo ) rnapairboth = (RNApair *)calloc( alloclen, sizeof( RNApair ) );
 
+	swaplist = NULL;
 	if( constraint )
 	{
+		if( ntarget < locnjob ) swaplist = calloc( njob, sizeof( char ) );
 		localhomshrink = (LocalHom ***)calloc( njob, sizeof( LocalHom ** ) );
 		for( i=0; i<njob; i++)
 		{
@@ -489,7 +498,6 @@ static void *athread( void *arg )
 		}
 	}
 
-
 	if( thread_no == 0 )
 	{
 		*ntrypt = 0;
@@ -532,6 +540,7 @@ static void *athread( void *arg )
 					effarr1_kozo, effarr2_kozo, effarr_kozo,
 					memlist, pairbuf,
 					localhomshrink,
+					swaplist,
 					smalldistmtx,
 					scoringmatrices,
 					eff1s, eff2s,
@@ -649,6 +658,7 @@ static void *athread( void *arg )
 			effarr1_kozo, effarr2_kozo, effarr_kozo,
 			memlist, pairbuf,
 			localhomshrink,
+			swaplist,
 			smalldistmtx,
 			scoringmatrices,
 			eff1s, eff2s,
@@ -694,6 +704,7 @@ static void *athread( void *arg )
 					effarr1_kozo, effarr2_kozo, effarr_kozo,
 					memlist, pairbuf,
 					localhomshrink,
+					swaplist,
 					smalldistmtx,
 					scoringmatrices,
 					eff1s, eff2s,
@@ -860,7 +871,10 @@ static void *athread( void *arg )
 
 //					shrinklocalhom( pair, int s1, int s2, localhomtable, localhomshrink );
 //					msshrinklocalhom( pair[0], pair[1], s1, s2, localhomtable, localhomshrink );
-					msshrinklocalhom_fast( memlist[0], memlist[1], localhomtable, localhomshrink );
+					if( ntarget < njob )
+						msshrinklocalhom_fast_target( memlist[0], memlist[1], localhomtable, localhomshrink, swaplist, targetmap ); // swaplist hitsuyou!!
+					else
+						msshrinklocalhom_fast_half( memlist[0], memlist[1], localhomtable, localhomshrink );
 					oimpmatchdouble = 0.0;
 					if( use_fft )
 					{
@@ -871,7 +885,7 @@ static void *athread( void *arg )
 						}
 						else if( alg == 'd' )
 						{
-							imp_match_init_strictD( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, 1 );
+							imp_match_init_strictD( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, swaplist, 1, memlist[0], memlist[1] );
 							if( rnakozo ) imp_rnaD( clus1, clus2, mseq1, mseq2, effarr1, effarr2, grouprna1, grouprna2, gapmap1, gapmap2, NULL );
 							for(  i=length-1; i>=0; i-- )
 							{
@@ -881,7 +895,7 @@ static void *athread( void *arg )
 						}
 						else
 						{
-							part_imp_match_init_strict( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, 1 );
+							part_imp_match_init_strict( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, swaplist, 1, memlist[0], memlist[1] );
 							if( rnakozo ) part_imp_rna( clus1, clus2, mseq1, mseq2, effarr1, effarr2, grouprna1, grouprna2, gapmap1, gapmap2, NULL );
 							for(  i=length-1; i>=0; i-- )
 							{
@@ -900,7 +914,7 @@ static void *athread( void *arg )
 						}
 						else
 						{
-							imp_match_init_strict( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, 1 );
+							imp_match_init_strict( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, swaplist, 1, memlist[0], memlist[1] );
 
 							fprintf( stderr, "not supported\n" );
 							exit( 1 );
@@ -1244,7 +1258,8 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 							 int **skipthisbranch,
                              int alloclen, LocalHom **localhomtable, 
 							 RNApair ***singlerna,
-							 int nkozo, char *kozoarivec )
+							 int nkozo, char *kozoarivec,
+							 int ntarget, int *targetmap, int *targetmapr )
 {
 	int i, j, k, l, iterate, ii, iu, ju;
 	int lin, ldf, length; 
@@ -1286,6 +1301,7 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 #endif
 	static char *indication1, *indication2;
 	static LocalHom ***localhomshrink = NULL;
+	static char *swaplist = NULL;
 	double impmatchdouble = 0.0;
 	double oimpmatchdouble = 0.0;
 	static int *gapmap1;
@@ -1363,8 +1379,10 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 		memlist = AllocateIntMtx( 2, locnjob );
 		if( rnakozo ) rnapairboth = (RNApair *)calloc( alloclen, sizeof( RNApair ) );
 
+		swaplist = NULL;
 		if( constraint )
 		{
+			if( ntarget < locnjob ) swaplist = calloc( njob, sizeof( char ) );
 			localhomshrink = (LocalHom ***)calloc( njob, sizeof( LocalHom ** ) );
 			for( i=0; i<njob; i++)
 			{
@@ -1387,7 +1405,10 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 		if( constraint )
 		{
 			counteff_simple( locnjob, topol, len, effarrforlocalhom );
-			calcimportance( locnjob, effarrforlocalhom, aseq, localhomtable );
+			if( ntarget < locnjob )
+				calcimportance_target( locnjob, ntarget, effarrforlocalhom, aseq, localhomtable, targetmap, targetmapr );
+			else
+				calcimportance_half( locnjob, effarrforlocalhom, aseq, localhomtable );
 		}
 
 		if( weight == 2 ) 
@@ -1398,10 +1419,12 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 				fprintf( stderr, "Not supported, weight=%d nkozo=%d.\n", weight, nkozo );
 			}
 		}
-		else if( weight == 4 )
+//		else if( weight == 4 )
+//		else if( weight == 4 || weight == 0 )
+		else if( locnjob > 2 && ( weight == 4 || weight == 0 ) )
 		{
 			treeCnv( stopol, locnjob, topol, len, branchWeight );
-			calcBranchWeight( branchWeight, locnjob, stopol, topol, len );
+			calcBranchWeight( branchWeight, locnjob, stopol, topol, len ); // IRU!!!
 		}
 	}
 
@@ -1484,6 +1507,8 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 			targ[i].collection_end = &collection_end;
 			targ[i].collection_start = &collection_start;
 			targ[i].tscorehistory_detail = tscorehistory_detail;
+			targ[i].ntarget = ntarget;
+			targ[i].targetmap = targetmap;
 			targ[i].finishpt = &finish;
 	
 			pthread_create( handle+i, NULL, athread, (void *)(targ+i) );
@@ -1591,10 +1616,12 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 	
 				if( weight == 2 )
 					countnode_int( locnjob, topol, node );
-				else if( weight == 4 )
+//				else if( weight == 4 ) 
+//				else if( weight == 4 || weight == 0 ) 
+				else if( locnjob > 2 && ( weight == 4 || weight == 0 ) ) 
 				{
 					treeCnv( stopol, locnjob, topol, len, branchWeight );
-					calcBranchWeight( branchWeight, locnjob, stopol, topol, len );
+					calcBranchWeight( branchWeight, locnjob, stopol, topol, len ); // IRU!!!
 				}
 				trap = fopen( "hat2", "w" );
 				if( !trap ) ErrorExit( "Cannot open hat2." );
@@ -1603,7 +1630,10 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 				if( constraint )
 				{
 					counteff_simple( locnjob, topol, len, effarrforlocalhom );
-					calcimportance( locnjob, effarrforlocalhom, aseq, localhomtable );
+					if( ntarget < locnjob )
+						calcimportance_target( locnjob, ntarget, effarrforlocalhom, aseq, localhomtable, targetmap, targetmapr );
+					else
+						calcimportance_half( locnjob, effarrforlocalhom, aseq, localhomtable );
 				}
 			}
 	
@@ -1663,7 +1693,7 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 					}
 
 					distFromABranch( locnjob, distarr, stopol, topol, len, l, k ); // ato de dou. weight 4 igai demo tsukaeruyou ni.
-					for( i=0; i<locnjob; i++ )
+//					for( i=0; i<locnjob; i++ ) // BUG??
 //					for( i=0; i<2; i++ ) for( j=0; j<locnjob; j++ ) pair[i][j] = 0;
 					OneClusterAndTheOther_fast( locnjob, memlist[0], memlist[1], &s1, &s2, pairbuf, topol, l, k, smalldistmtx, distmtx, distarr );
 
@@ -1795,7 +1825,10 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 						if( constraint )
 						{
 //							msshrinklocalhom( pair[0], pair[1], s1, s2, localhomtable, localhomshrink );
-							msshrinklocalhom_fast( memlist[0], memlist[1], localhomtable, localhomshrink );
+							if( ntarget < locnjob )
+								msshrinklocalhom_fast_target( memlist[0], memlist[1], localhomtable, localhomshrink, swaplist, targetmap );
+							else
+								msshrinklocalhom_fast_half( memlist[0], memlist[1], localhomtable, localhomshrink );
 							oimpmatchdouble = 0.0;
 							if( use_fft )
 							{
@@ -1806,7 +1839,7 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 								}
 								else
 								{
-									part_imp_match_init_strict( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, 1 );
+									part_imp_match_init_strict( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, swaplist, 1, memlist[0], memlist[1] );
 									if( rnakozo ) part_imp_rna( clus1, clus2, mseq1, mseq2, effarr1, effarr2, grouprna1, grouprna2, gapmap1, gapmap2, NULL );
 									for(  i=length-1; i>=0; i-- ) oimpmatchdouble += (double)part_imp_match_out_sc( i, i );
 								}
@@ -1820,7 +1853,7 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 								}
 								else
 								{
-									imp_match_init_strict( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, 1 );
+									imp_match_init_strict( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, swaplist, 1, memlist[0], memlist[1] );
 									fprintf( stderr, "not supported\n" );
 									exit( 1 );
 								}
@@ -1876,7 +1909,10 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 								intergroup_score( mseq1, mseq2, effarr1, effarr2, clus1, clus2, length, &tmpdouble ); // gappick mae denaito dame
 	
 //							shrinklocalhom( pair, s1, s2, localhomtable, localhomshrink );
-							msshrinklocalhom_fast( memlist[0], memlist[1], localhomtable, localhomshrink );
+							if( ntarget < locnjob )
+								msshrinklocalhom_fast_target( memlist[0], memlist[1], localhomtable, localhomshrink, swaplist, targetmap );
+							else
+								msshrinklocalhom_fast_half( memlist[0], memlist[1], localhomtable, localhomshrink );
 		//					weightimportance4( clus1, clus2,  effarr1, effarr2, localhomshrink ); // >>>
 							oimpmatchdouble = 0.0;
 							if( use_fft )
@@ -1888,7 +1924,7 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 								}
 								else if( alg == 'd' )
 								{
-									imp_match_init_strictD( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, 1 );
+									imp_match_init_strictD( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, swaplist, 1, memlist[0], memlist[1] );
 	
 									for(  i=length-1; i>=0; i-- )
 									{
@@ -1898,7 +1934,7 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 								}
 								else
 								{
-									part_imp_match_init_strict( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, 1 );
+									part_imp_match_init_strict( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, swaplist, 1, memlist[0], memlist[1] );
 									if( rnakozo ) part_imp_rna( clus1, clus2, mseq1, mseq2, effarr1, effarr2, grouprna1, grouprna2, gapmap1, gapmap2, NULL );
 									for(  i=length-1; i>=0; i-- )
 									{
@@ -1917,7 +1953,7 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 								}
 								else
 								{
-									imp_match_init_strict( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, 1 );
+									imp_match_init_strict( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, swaplist, 1, memlist[0], memlist[1] );
 	
 									fprintf( stderr, "not supported\n" );
 									exit( 1 );
@@ -1970,7 +2006,10 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 						{
 							oimpmatchdouble = 0.0;
 //							shrinklocalhom( pair, s1, s2, localhomtable, localhomshrink );
-							msshrinklocalhom_fast( memlist[0], memlist[1], localhomtable, localhomshrink );
+							if( ntarget < locnjob )
+								msshrinklocalhom_fast_target( memlist[0], memlist[1], localhomtable, localhomshrink, swaplist, targetmap );
+							else
+								msshrinklocalhom_fast_half( memlist[0], memlist[1], localhomtable, localhomshrink );
 							if( use_fft )
 							{
 								if( alg == 'Q' )
@@ -1980,12 +2019,12 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 								}
 								else if( alg == 'd' )
 								{
-									imp_match_init_strictD( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, 1 );
+									imp_match_init_strictD( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, swaplist, 1, memlist[0], memlist[1] );
 									if( rnakozo ) imp_rnaD( clus1, clus2, mseq1, mseq2, effarr1, effarr2, grouprna1, grouprna2, gapmap1, gapmap2, NULL );
 								}
 								else
 								{
-									part_imp_match_init_strict( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, 1 );
+									part_imp_match_init_strict( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, swaplist, 1, memlist[0], memlist[1] );
 									if( rnakozo ) part_imp_rna( clus1, clus2, mseq1, mseq2, effarr1, effarr2, grouprna1, grouprna2, gapmap1, gapmap2, NULL );
 								}
 							}
@@ -1998,7 +2037,7 @@ int TreeDependentIteration( int locnjob, char **name, int nlen[M],
 								}
 								else
 								{
-									imp_match_init_strict( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, 1 );
+									imp_match_init_strict( NULL, clus1, clus2, length, length, mseq1, mseq2, effarr1, effarr2, effarr1_kozo, effarr2_kozo, localhomshrink, swaplist, 1, memlist[0], memlist[1] );
 									fprintf( stderr, "Not supported\n" );
 									exit( 1 );
 								}
